# Copyright (c) 2017 Riverbed Technology, Inc.
#
# This software is licensed under the terms and conditions of the MIT License
# accompanying the software ("License").  This software is distributed "AS IS"
# as set forth in the License.


import os

import copy
import logging
import pprint

from django import forms

from steelscript.appfwk.apps.devices.devicemanager import DeviceManager
from steelscript.common import datetime_to_seconds
from steelscript.common.exceptions import RvbdHTTPException
from steelscript.netshark.appfwk.datasources.netshark import NetSharkTable, \
    fields_add_bpf_filterexpr, NetSharkQuery, NetSharkColumn
from steelscript.netshark.appfwk.datasources.netshark_pcap import \
    PcapDownloadTable

from steelscript.appfwk.apps.devices.forms import fields_add_device_selection
from steelscript.appfwk.apps.datasource.modules.analysis import \
    AnalysisTable, AnalysisQuery
from steelscript.appfwk.apps.datasource.models import TableField, Table
from steelscript.appfwk.apps.datasource.forms import \
    fields_add_time_selection, IDChoiceField, fields_add_resolution
from steelscript.appfwk.libs.fields import Function
from steelscript.wireshark.appfwk.datasources.wireshark_source import \
    fields_add_filterexpr
from steelscript.appfwk.apps.jobs import QueryComplete, QueryContinue
from steelscript.appfwk.apps.jobs.models import Job


logger = logging.getLogger(__name__)


def check_netshark_file(netshark, fname, remove=False):
    """Check for existance of file on NetShark, optionally removing."""
    if netshark.exists(fname):
        if remove:
            logger.info("Removing existing file: %s" % fname)
            f = netshark.get_file(fname)
            f.remove()
        return True
    return False


def netshark_source_choices(form, id_, field_kwargs, params):
    """Query netshark for available capture jobs / trace clips."""
    # simplified clone from base netshark datasource that allows for
    # custom field names

    netshark_device = form.get_field_value(params['field'], id_)
    if netshark_device == '':
        choices = [('', '<No netshark device>')]
    else:
        netshark = DeviceManager.get_device(netshark_device)

        choices = []

        for job in netshark.get_capture_jobs():
            choices.append((job.source_path, job.name))

        for clip in netshark.get_clips():
            choices.append((clip.source_path, 'Clip: ' + clip.description))

    field_kwargs['choices'] = choices


def netshark_msa_file_choices(form, id_, field_kwargs, params):
    """Query netshark for available MSA files."""

    netshark_device = form.get_field_value('netshark_device', id_)
    if netshark_device == '':
        choices = [('', '<No netshark device>')]
    else:
        netshark = DeviceManager.get_device(netshark_device)

        choices = []

        # TODO - this enables automatic refresh
        # whenever the report or criteria get reloaded, but
        # it will cause some extra load when the report runs
        # since each time a TableForm gets created all the files
        # will be re-queried
        for f in netshark.get_files(force_refetch=True):
            if hasattr(f, 'list_linked_files'):
                choices.append((f.source_path, 'MSA File: ' + f.path))

    field_kwargs['choices'] = choices


class MSADownloadTable(AnalysisTable):
    class Meta:
        proxy = True
        app_label = 'steelscript.netshark.appfwk'

    _query_class = 'MSADownloadQuery'

    TABLE_OPTIONS = {'msa_folder': None,
                     'overwrite_folder': True}

    @classmethod
    def create(cls, name, **kwargs):
        download_table = PcapDownloadTable.create(name + '-download')
        kwargs['related_tables'] = {'download_table': download_table}
        return super(MSADownloadTable, cls).create(name, **kwargs)

    def post_process_table(self, field_options):
        fields_add_device_selection(self, keyword='netshark_device_src',
                                    label='NetShark Source', module='netshark',
                                    enabled=True)
        fields_add_device_selection(self, keyword='netshark_device_dst',
                                    label='NetShark Dest', module='netshark',
                                    enabled=True)

        fields_add_device_selection(self, keyword='netshark_device_upload',
                                    label='NetShark MSA', module='netshark',
                                    enabled=True)
        TableField.create(
            keyword='netshark_source_name_src',
            label='Source Capture Job',
            obj=self,
            field_cls=forms.ChoiceField,
            field_kwargs={'widget_attrs': {'class': 'form-control'}},
            parent_keywords=['netshark_device_src'],
            dynamic=True,
            pre_process_func=Function(netshark_source_choices,
                                      {'field': 'netshark_device_src'})
        )
        TableField.create(
            keyword='netshark_source_name_dst',
            label='Dest Capture Job',
            obj=self,
            field_cls=forms.ChoiceField,
            field_kwargs={'widget_attrs': {'class': 'form-control'}},
            parent_keywords=['netshark_device_dst'],
            dynamic=True,
            pre_process_func=Function(netshark_source_choices,
                                      {'field': 'netshark_device_dst'})
        )

        fields_add_time_selection(self, show_start=True,
                                  initial_start_time='now-1m',
                                  show_end=True,
                                  show_duration=False)

        fields_add_filterexpr(obj=self)


class MSADownloadQuery(AnalysisQuery):

    def analyze(self, jobs=None):

        download_table = Table.from_ref(
            self.table.options.related_tables['download_table']
        )

        # Create source and destination download jobs
        depjobs = {}

        c = self.job.criteria
        sharks = [
            ('1-source', c.netshark_device_src, c.netshark_source_name_src),
            ('2-dest', c.netshark_device_dst, c.netshark_source_name_dst)
        ]

        for shark in sharks:
            sc = copy.copy(c)
            name, device, source = shark
            sc.netshark_device = device
            sc.netshark_source_name = source
            sc.segment = name

            job = Job.create(table=download_table, criteria=sc,
                             update_progress=True, parent=self.job)
            logger.debug("Created %s: %s download job with criteria %s"
                         % (job, name, sc))
            depjobs[job.id] = job

        return QueryContinue(self.collect, depjobs)

    def collect(self, jobs=None):
        logger.info('%s: MSADownload.collect: %s' % (self, jobs))

        c = self.job.criteria
        s = DeviceManager.get_device(c.netshark_device_upload)

        if self.table.options.msa_folder is None:
            upload_dir = ('/%s/msa-%s' %
                          (s.auth.username,
                           datetime_to_seconds(c.endtime)))
        else:
            upload_dir = self.table.options.msa_folder

        # check upload dir and delete if option set
        try:
            s.create_dir(upload_dir)
        except RvbdHTTPException:
            if self.table.options.overwrite_folder:
                logger.info('MSA Directory %s already created, '
                            'deleting and creating a new one.' % upload_dir)
                d = s.get_dir(upload_dir)
                d.remove()
                s.create_dir(upload_dir)
            else:
                logger.info('MSA Directory %s already created, skipping'
                            % upload_dir)

        # upload pcaps to upload dir, overwriting if needed
        filepaths = []

        for jid, job in jobs.iteritems():
            localfile = job.data()['filename'][0]

            # example naming: "1-source-6-jobs_00001E3"
            fname = '{}-{}-{}'.format(
                job.criteria.segment,
                job.criteria.netshark_device,
                job.criteria.netshark_source_name.replace('/', '_')
            )

            remotefile = os.path.join(upload_dir, fname)
            check_netshark_file(s, remotefile, remove=True)

            logger.info('Uploading new PCAP %s to %s' % (localfile,
                                                         remotefile))
            s.upload_trace_file(remotefile, localfile)

            filepaths.append((remotefile, localfile))

        # create msa from those files
        filepaths.sort()
        flist = [s.get_file(rfile) for rfile, _ in filepaths]

        msafile = os.path.join(upload_dir, 'msa_file.pvt')
        check_netshark_file(s, msafile, remove=True)

        # create the aggregated file and initiate a timeskew calculation
        msa = s.create_multisegment_file(msafile, flist)

        # pick arbitrary number 10,000 for number of pkts
        # to use for timeskew calculation
        msa.calculate_timeskew(10000)

        result = ['<strong>Done</strong> - PCAPs successfully downloaded '
                  'from source NetSharks and uploaded for analysis']
        result.append('')
        for remote, local in filepaths:
            result.append(
                '<strong>Uploaded file path:</strong> %s' % remote
            )
        result.append('')

        config = pprint.pformat(msa.get_info()).splitlines()
        result.append('<strong>MSA file created with config:</strong> '
                      '<br>'
                      '<pre>'
                      '{}'
                      '</pre>'.format('<br>'.join(config)))
        return QueryComplete(['<br>'.join(result)])


class MSATable(NetSharkTable):
    class Meta:
        proxy = True
        app_label = 'steelscript.netshark.appfwk'

    _query_class = 'MSAQuery'

    TABLE_OPTIONS = {'aggregated': False,
                     'include_files': False,
                     'include_interfaces': False,
                     'include_persistent': False,
                     'include_filter': True,       # included by default
                     'include_bpf_filter': False,
                     'show_entire_msa': True,
                     }

    def post_process_table(self, field_options):
        duration = field_options['duration']
        if isinstance(duration, int):
            duration = "{}m".format(duration)

        fields_add_device_selection(self, keyword='netshark_device',
                                    label='NetShark', module='netshark',
                                    enabled=True)

        func = Function(netshark_msa_file_choices,
                        self.options)
        TableField.create(keyword='netshark_source_name', label='Source',
                          obj=self,
                          field_cls=IDChoiceField,
                          field_kwargs={
                              'widget_attrs': {'class': 'form-control'}
                          },
                          parent_keywords=['netshark_device'],
                          dynamic=True,
                          pre_process_func=func)

        fields_add_time_selection(self,
                                  initial_duration=duration,
                                  durations=field_options['durations'])
        fields_add_resolution(self,
                              initial=field_options['resolution'],
                              resolutions=field_options['resolutions'])

        if self.options.include_filter:
            fields_add_filterexpr(self)

        if self.options.include_bpf_filter:
            fields_add_bpf_filterexpr(self)

        if self.options.show_entire_msa:
            TableField.create(keyword='entire_msa', obj=self,
                              field_cls=forms.BooleanField,
                              label='Entire MSA (ignore start/end times)',
                              initial=True,
                              required=False)


class MSAQuery(NetSharkQuery):

    def run(self):
        """Run MSA Table"""
        criteria = self.job.criteria

        # if `entire_msa` option set, change start/end to null values
        if criteria.entire_msa:
            criteria.starttime = None
            criteria.endtime = None

        return super(MSAQuery, self).run()



#!/usr/bin/env python

# Copyright (c) 2015 Riverbed Technology, Inc.
#
# This software is licensed under the terms and conditions of the MIT License
# accompanying the software ("License").  This software is distributed "AS IS"
# as set forth in the License.


"""
This script shows how to create a Capture Job in the simplest possible way:
only the required job parameters are set, while all the optional parameters
keep their default value.
"""

from steelscript.netshark.core.app import NetSharkApp
from steelscript.common.datautils import bytes2human


class CreateJobApp(NetSharkApp):
    def add_options(self, parser):
        super(CreateJobApp, self).add_options(parser)
        parser.add_option('-j', '--jobname', default=None, help='job name')
        parser.add_option('--capture-port', help='capture port')
        parser.add_option('-s', '--size', help='size', default=None)

    def main(self):
        # Make the user pick a name for the job
        name = self.options.jobname
        if name is None:
            name = raw_input('Job name? ')

        # Make the user pick the capture port
        try:
            ifc = self.netshark.get_interface_by_name(self.options.capture_port)
        except KeyError:
            print 'Capture Ports:'
            interfaces = [ifc for ifc in self.netshark.get_interfaces()
                          if 'virtual_info' not in ifc.__dict__['data']]

            for i, ifc in enumerate(interfaces):
                print '\t{0}. {1} {2}'.format(i + 1, ifc.id, ifc.description)

            while 1:
                idx = raw_input('Port number(1-{0})? '.format(len(interfaces)))
                try:
                    ifc = interfaces[int(idx) - 1]
                    break
                except IndexError:
                    print 'Bad index try again'

        # Make the user pick the job size
        size = self.options.size
        if size is None:
            stats = self.netshark.get_stats()
            storage = stats['storage']['packet_storage']
            print 'Storage size is {0}, {1} available'.format(bytes2human(storage['total']),
                                                              bytes2human(storage['unused']))
            size = raw_input('Job size, greater than 256MB (e.g. 1.1GB, 512M, 20%)? ')

        job = self.netshark.create_job(ifc, name, size)
        print 'Capture Job successfully created with the following properties:'
        print ''
        print 'ID: %s' % job.id
        print 'Current State: %s' % job.get_state()
        print 'Name: %s' % job.name
        print 'Interface: %s' % job.interface
        print 'Source Path: %s' % job.source_path
        print 'Size limit: %s' % job.size_limit
        print 'Current size on disk: %s' % job.size_on_disk
        print ''


if __name__ == '__main__':
    CreateJobApp().run()



# Copyright (c) 2015 Riverbed Technology, Inc.
#
# This software is licensed under the terms and conditions of the MIT License
# accompanying the software ("License").  This software is distributed "AS IS"
# as set forth in the License.


import steelscript.appfwk.apps.report.modules.c3 as c3
from steelscript.appfwk.apps.report.models import Report
import steelscript.appfwk.apps.report.modules.tables as tables
from steelscript.netshark.appfwk.datasources.netshark import \
    NetSharkTable

#
# Define a NetShark Report and Table
#
report = Report.create('NetShark TCP Errors', position=10)
report.add_section()

tcp = NetSharkTable.create(name='TCPErrors', aggregated=True)

tcp.add_column('error_type', label='TCP Error Type', iskey=True,
               extractor='tcp.error_type', datatype='string')
tcp.add_column('errors', label='TCP Errors', sortdesc=True,
               extractor='tcp.errors', datatype='integer', operation='sum',
               default_value=0)

report.add_widget(c3.BarWidget, tcp,
                  'TCP Errors', width=6, height=400)
report.add_widget(tables.TableWidget, tcp,
                  'TCP Errors Table', width=6, height=400)



#!/usr/bin/env python

# Copyright (c) 2015 Riverbed Technology, Inc.
#
# This software is licensed under the terms and conditions of the MIT License
# accompanying the software ("License").  This software is distributed "AS IS"
# as set forth in the License.



"""
This script connects to a NetShark appliance, collects a bounch of information
about it, and prints it the screen.
"""

from steelscript.netshark.core.app import NetSharkApp
from steelscript.common.datautils import bytes2human


class NetSharkInfo(NetSharkApp):

    def main(self):
        # Print the high level netshark info
        print 'APPLIANCE INFO:'

        info = self.netshark.get_serverinfo()
        print '\tAppliance Version: ' + info['version']
        print '\tAppliance Hostname: ' + info['hostname']
        print '\tUptime: ' + str(info['uptime'])
        print '\tWeb UI TCP Port: {0}'.format(info['webui_port'])

        stats = self.netshark.get_stats()
        print '\tPacket Storage: {0} total, {1} free, status:{2}'.format(
            bytes2human(stats['storage']['packet_storage']['total']),
            bytes2human(stats['storage']['packet_storage']['unused']),
            stats['storage']['packet_storage']['status'])
        print '\tIndex Storage: {0} total, {1} free, status:{2}'.format(
            bytes2human(stats['storage']['os_storage']['index_storage']['total']),
            bytes2human(stats['storage']['os_storage']['index_storage']['unused']),
            stats['storage']['os_storage']['status'])
        print '\tOS File System: {0} total, {1} free, status:{2}'.format(
            bytes2human(stats['storage']['os_storage']['disk_storage']['total']),
            bytes2human(stats['storage']['os_storage']['disk_storage']['unused']),
            stats['storage']['os_storage']['status'])
        print '\tmemory: {0} total, {1} free, status:{2}'.format(
            bytes2human(stats['memory']['total']),
            bytes2human(stats['memory']['available']),
            stats['memory']['status'])

        # Print the list of interfaces
        print 'INTERFACES:'
        for i in self.netshark.get_interfaces():
            print '\t{0} (OS name: {1})'.format(i, i.name)

        # Print the list of trace files
        print 'TRACE FILES:'
        for f in self.netshark.get_files():
            print '\t{0} ({1} bytes, created: {2})'.format(f, f.size, f.created)

        # Print the list of capture jobs
        print 'JOBS:'
        jobs = self.netshark.get_capture_jobs()
        for j in jobs:
            print '\t{0} (size: {1}, src interface: {2})'.format(j, j.size_limit, j.interface)

        # Print the list of trace clips
        print 'TRACE CLIPS:'
        for c in self.netshark.get_clips():
            if c.description == "":
                print '\t{0} ({1} bytes)'.format(c, c.size)
            else:
                print '\t{0}, {1} ({2} bytes)'.format(c.description, c, c.size)

        # Print the list of open views
        print 'OPEN VIEWS:'
        for view in self.netshark.get_open_views():
            print '\t{0}'.format(view.handle)
            for output in view.all_outputs():
                print '\t\t{0}'.format(output.id)


if __name__ == '__main__':
    NetSharkInfo().run()



# Copyright (c) 2015 Riverbed Technology, Inc.
#
# This software is licensed under the terms and conditions of the MIT License
# accompanying the software ("License").  This software is distributed "AS IS"
# as set forth in the License.

import datetime
import pytz

from django.db import models


class NetSharkViews(models.Model):

    class Meta:
        app_label = 'steelscript.netshark.appfwk'

    netsharkname = models.CharField(max_length=300)
    viewname = models.CharField(max_length=300)
    viewid = models.CharField(max_length=12)

    # Timestamp when the view was last used
    lastused = models.DateTimeField(auto_now_add=True)

    @classmethod
    def find_by_name(cls, netshark, viewname):
        # First see if this is already cached
        nvs = NetSharkViews.objects.filter(
            netsharkname=netshark.host, viewname=viewname)

        if len(nvs) > 0:
            try:
                view = netshark.get_open_view_by_handle(nvs[0].viewid)
                nvs[0].lastused = datetime.datetime.now(tz=pytz.utc)
                nvs[0].save()
                return view
            except KeyError:
                # If the view was not found, may need to refresh, so
                # continue on below
                view = None

        # Delete old views for this netshark
        NetSharkViews.objects.filter(netsharkname=netshark.host).delete()

        # No match, ask the shark for the complete list of views
        views = netshark.get_open_views()

        # Insert those views that have a title
        match = None
        for view in views:
            if ('info' in view.config and
                    'title' in view.config['info'] and
                    view.config['info']['title']):

                title = view.config['info']['title']
                nv = NetSharkViews(netsharkname=netshark.host,
                                   viewname=title,
                                   viewid=view.handle)
                nv.save()
                if title == viewname:
                    match = view

        return match



# Copyright (c) 2015 Riverbed Technology, Inc.
#
# This software is licensed under the terms and conditions of the MIT License
# accompanying the software ("License").  This software is distributed "AS IS"
# as set forth in the License.

import steelscript.appfwk.apps.report.modules.tables as tables
import steelscript.netshark.appfwk.datasources.netshark_scanner_source as \
    scanner

from steelscript.appfwk.apps.report.models import Report
from steelscript.netshark.appfwk.datasources.netshark import NetSharkTable

# Import the datasource module for this plugin (if needed)


report = Report.create("NetShark Scanner", field_order=['endtime', 'duration'],
                       hidden_fields=['netshark_device',
                                      'netshark_source_name', 'resolution'])
report.add_section()

# Create base table
shark_bytes_table = NetSharkTable.create(name='shark_bytes', aggregated=True)
shark_bytes_table.add_column('generic_bytes', label='Bytes', iskey=False,
                             extractor='generic.bytes', operation='sum')

# Make
table = scanner.SharksTable.create(name='sharks',
                                   basetable=shark_bytes_table)
table.add_column('name', "Name", datatype='string')
table.add_column('host', "Host", datatype='string')
table.add_column('capjob', "Capture Job", datatype='string')
table.add_column('bytes', "Bytes")

report.add_widget(tables.TableWidget, table, "Shark Capture Jobs Found",
                  width=12, height=200)



#!/usr/bin/env python

# Copyright (c) 2015 Riverbed Technology, Inc.
#
# This software is licensed under the terms and conditions of the MIT License
# accompanying the software ("License").  This software is distributed "AS IS"
# as set forth in the License.

import os

from steelscript.netshark.core.app import NetSharkApp


class UploadPcap(NetSharkApp):
    def add_options(self, parser):
        super(UploadPcap, self).add_options(parser)
        parser.add_option('--filepath',
                          help='path to pcap tracefile to upload')
        parser.add_option('--destname', default=None,
                          help='location to store on server, defaults to '
                               '<username>/<basename of filepath>')

    def validate_args(self):
        super(UploadPcap, self).validate_args()

        basedir = '/' + self.options.username

        if not self.options.destname:
            dst = os.path.join(basedir,
                               os.path.basename(self.options.filepath))
            self.options.destname = dst

    def main(self):
        self.netshark.upload_trace_file(
            path=self.options.destname,
            local_file=self.options.filepath
        )


if __name__ == "__main__":
    app = UploadPcap()
    app.run()



# Copyright (c) 2015 Riverbed Technology, Inc.
#
# This software is licensed under the terms and conditions of the MIT License
# accompanying the software ("License").  This software is distributed "AS IS"
# as set forth in the License.


from steelscript.netshark.core import NetShark


def new_device_instance(*args, **kwargs):
    # Used by DeviceManger to create a NetShark instance
    shark = NetShark(*args, **kwargs)
    return shark



#!/usr/bin/env python

# Copyright (c) 2015 Riverbed Technology, Inc.
#
# This software is licensed under the terms and conditions of the MIT License
# accompanying the software ("License").  This software is distributed "AS IS"
# as set forth in the License.


"""
This script can be used to view the amount of traffic reaching a given
NetShark. You can also provide a traffic expression and/or time filter to
limit the traffic counted in the results. A CSV config file may also be
provided to view results for multiple NetSharks at a time.
"""

import operator
import csv

from datetime import datetime

from steelscript.common.app import Application
from steelscript.netshark.core import NetShark
from steelscript.netshark.core.filters import NetSharkFilter, TimeFilter
from steelscript.netshark.core.types import Value
from steelscript.common.datautils import Formatter
from steelscript.common.service import UserAuth
from steelscript.common.timeutils import TimeParser


class SharkFinderApp(Application):
    def add_options(self, parser):
        super(SharkFinderApp, self).add_options(parser)

        parser.add_option('-H', '--host',
                          help='NetShark hostname or IP address')
        parser.add_option('-u', '--username', help="Username to connect with")
        parser.add_option('-p', '--password', help="Password to connect with")
        parser.add_option('-f', '--file', help=("CSV config file in format "
                                                "netshark, username, "
                                                "password"))

        parser.add_option('--starttime',
                          help=("Analyze traffic starting at this time "
                                "(timestamp format)"))
        parser.add_option('--endtime',
                          help=("Analyze traffic ending at this time "
                                "(timestamp format)"))
        parser.add_option(
            '--timerange',
            help=("Time range to analyze (defaults to \"last 1 hour\") "
                  "other valid formats are: \"4/21/13 4:00 to 4/21/13 5:00\" "
                  "or \"16:00:00 to 21:00:04.546\""))

        parser.add_option('--filter', dest='filters', action='append',
                          default=[],
                          help=("Filter to apply to analysis; can be repeated "
                                "as many times as desired"))

    def validate_args(self):
        super(SharkFinderApp, self).validate_args()

        if (self.options.timerange is not None and
                (self.options.starttime is not None
                 or self.options.endtime is not None)):
            self.parser.error("Can't use --timerange and --starttime/"
                              "--endtime together.")

        if self.options.file is not None and self.options.host is not None:
            self.parser.error(("When using a CSV file, you cannot specify a "
                               "NetShark on the command line."))
        elif self.options.host is None and self.options.file is None:
            self.parser.error("You must either specify a NetShark on the "
                              "command line or a config file with --file.")

    def get_jobs_bytes(self, shark, filters):
        """ Takes a shark and filters; returns an array of
            [(job_name, bytes), ...] or None if the shark returned no data
        """
        out = []
        for job in shark.get_capture_jobs():
            columns = [Value(shark.columns.generic.bytes)]

            with shark.create_view(job, columns, filters=filters) as view:
                data = view.get_data(aggregated=True)

            if data:
                job_bytes = data[0]['vals'][0][0]

                out.append((job.name, job_bytes))
        return out

    def get_csv_sharks_info(self, filename):
        with open(filename) as configfile:
            return [row for row in csv.reader(configfile)]

    def main(self):
        if self.options.timerange is not None:
            try:
                timefilter = TimeFilter.parse_range(self.options.timerange)
            except ValueError:
                print "Could not parse time filter expression."
                return
        elif (self.options.starttime is not None or
              self.options.endtime is not None):
            timeparser = TimeParser()

            if self.options.starttime is None:
                start_time = datetime.min
            else:
                try:
                    start_time = timeparser.parse(self.options.starttime)
                except ValueError:
                    print "Could not parse start timestamp"
                    return

            if self.options.endtime is None:
                end_time = datetime.now()
            else:
                try:
                    end_time = timeparser.parse(self.options.endtime)
                except ValueError:
                    print "Could not parse end timestamp"
                    return
            timefilter = TimeFilter(start_time, end_time)
        else:
            timefilter = None

        filters = [NetSharkFilter(f) for f in self.options.filters]
        if timefilter is not None:
            filters.append(timefilter)

        if self.options.file is None:
            sharks_info = [[self.options.host, self.options.username,
                            self.options.password]]
        else:
            sharks_info = self.get_csv_sharks_info(self.options.file)

        out_table = []
        for host, username, password in sharks_info:
            shark = NetShark(host, auth=UserAuth(username, password))

            jobs_bytes = self.get_jobs_bytes(shark, filters)
            if not jobs_bytes:
                print "(No data returned from NetShark {0}.)".format(host)
            else:
                for job_name, job_bytes in self.get_jobs_bytes(shark, filters):
                    out_table.append([host, job_name, job_bytes])

        if not out_table:
            print "No data found by any NetShark."
        else:
            out_table_sorted = sorted(out_table, reverse=True,
                                      key=operator.itemgetter(2))

            heads = ["NetShark", "Job", "Total bytes"]
            Formatter.print_table(out_table_sorted, heads)

if __name__ == '__main__':
    SharkFinderApp().run()



# Copyright (c) 2015 Riverbed Technology, Inc.
#
# This software is licensed under the terms and conditions of the MIT License
# accompanying the software ("License").  This software is distributed "AS IS"
# as set forth in the License.

import steelscript.appfwk.apps.report.modules.c3 as c3
from steelscript.appfwk.apps.report.models import Report
import steelscript.appfwk.apps.report.modules.tables as tables

from steelscript.netshark.appfwk.datasources.netshark import NetSharkTable
#
# Define a NetShark Report and Table
#

report = Report.create('NetShark', position=10)

report.add_section()

# NetShark Time Series
t = NetSharkTable.create(name='Total Traffic Bits',
                         duration=1, resolution='1sec', aggregated=False)

t.add_column('time', label='Time', iskey=True,
             extractor='sample_time', datatype='time')
t.add_column('generic_bits', label='Bits', iskey=False,
             extractor='generic.bits', operation='sum')

report.add_widget(c3.TimeSeriesWidget, t, 'Overall Bandwidth (Bits)',
                  width=12)

# Table for NetShark
t = NetSharkTable.create(name='Packet Traffic', duration=1, aggregated=True,
                         rows=100)

t.add_column('ip_src', label='Source IP', iskey=True, extractor='ip.src',
             datatype='string')
t.add_column('ip_dst', label='Dest IP', iskey=True, extractor='ip.dst',
             datatype='string')
t.add_column('generic_bits', label='Bits', iskey=False,
             extractor='generic.bits', operation='sum', units='B')
t.add_column('generic_packets', label='Packets', iskey=False, sortdesc=True,
             extractor='generic.packets', operation='sum')

report.add_widget(tables.TableWidget, t, 'Top 100 Conversations', width=12)

# TCP/UDP Traffic tables
t = NetSharkTable.create(name='Traffic by TCP/UDP', duration=1,
                         aggregated=False)

t.add_column('time', label='Time (ns)', iskey=True,
             extractor='sample_time', datatype='time')
t.add_column('udp_bits', label='UDP Bits', iskey=False,
             extractor='udp.bits', operation='sum', default_value=0)
t.add_column('tcp_bits', label='TCP Bits', iskey=False,
             extractor='tcp.bits', operation='sum', default_value=0)

report.add_widget(c3.TimeSeriesWidget, t, 'Traffic By Type (Bits)', width=12)



# Copyright (c) 2015 Riverbed Technology, Inc.
#
# This software is licensed under the terms and conditions of the MIT License
# accompanying the software ("License").  This software is distributed "AS IS"
# as set forth in the License.



from steelscript.netshark.core._api_helpers import APIGroup, APITimestampFormat
import urllib


class API4Group(APIGroup):

    base_headers = {}

    def _xjtrans(self, urlpath, method, data, as_json,
                 timestamp_format=APITimestampFormat.NANOSECOND,
                 params=None,
                 custom_headers=None):
        """Issue the given API request using either JSON or XML
        (dictated by the as_json parameter)."""
        self.add_base_header('X-RBT-High-Precision-Timestamp-Format', timestamp_format)

        # XXXWP Changing the method so that the caller can specify some extra headers
        headers = dict(self.base_headers)
        if isinstance(custom_headers, dict):
            headers.update(custom_headers)

        # we are dealing with a url so let's sanitize it.
        # this may break code but at least prevents steelscript to send insane urls to the server
        # define insane: url that contains spaces
        urlpath = urllib.quote(urlpath)

        if as_json:
            return self.shark.conn.json_request(method, self.uri_prefix + urlpath,
                                                body=data, params=params, extra_headers=headers)
        else:
            return self.shark.conn.xml_request(method, self.uri_prefix + urlpath,
                                               body=data,
                                               params=params, extra_headers=headers)

    def add_base_header(self, key, value=""):
        if isinstance(key, basestring):
            self.base_headers[key] = value

    def remove_base_header(self, key):
        if isinstance(key, basestring):
            self.base_headers.pop(key, None)


class Common(API4Group):
    def get_services(self, as_json=True, timestamp_format=APITimestampFormat.NANOSECOND):
        """Return the set of services running on this system
        """
        return self._xjtrans("/services", "GET", None, as_json, timestamp_format)

    def get_auth_info(self, as_json=True, timestamp_format=APITimestampFormat.NANOSECOND):
        """Return authentication information necessary to properly log in to the system.
        """
        return self._xjtrans("/auth_info", "GET", None, as_json, timestamp_format)

    def login(self, credentials, as_json=True, timestamp_format=APITimestampFormat.NANOSECOND):
        """Log in to the system using the given credentials. Returns the session cookie for use in subsequent calls.
        """
        return self._xjtrans("/login", "POST", credentials, as_json, timestamp_format)

    def logout(self, as_json=True, timestamp_format=APITimestampFormat.NANOSECOND):
        """Log out from the system by invalidating the session cookie
        """
        headers = {'connection':'close'}
        return self._xjtrans("/logout", "POST", {}, as_json, timestamp_format, custom_headers=headers)

    def ping(self, as_json=True, timestamp_format=APITimestampFormat.NANOSECOND):
        return self._xjtrans("/ping", "GET", None, as_json, timestamp_format)

    def info(self):
        return self._xjtrans("/info", "GET", None, True, APITimestampFormat.NANOSECOND)

class Settings(API4Group):
    def get_basic(self, as_json=True, timestamp_format=APITimestampFormat.NANOSECOND):
        """ Retrieves the basic configuration settings """
        return self._xjtrans("/settings/basic", "GET", None, as_json, timestamp_format)

    def update_basic(self, config, as_json=True, timestamp_format=APITimestampFormat.NANOSECOND):
        """ Updates the basic configuration settings """
        return self._xjtrans("/settings/basic", "PUT", config, as_json, timestamp_format)

    def get_auth(self, as_json=True, timestamp_format=APITimestampFormat.NANOSECOND):
        """ Retrieves the authentication settings """
        return self._xjtrans("/settings/auth", "GET", None, as_json, timestamp_format)

    def update_auth(self, config, as_json=True, timestamp_format=APITimestampFormat.NANOSECOND):
        """ Updates the authentication settings """
        return self._xjtrans("/settings/auth", "PUT", config, as_json, timestamp_format)

    def get_audit(self, as_json=True, timestamp_format=APITimestampFormat.NANOSECOND):
        """ Retrieves the audit settings """
        return self._xjtrans("/settings/audit", "GET", None, as_json, timestamp_format)

    def update_audit(self, config, as_json=True, timestamp_format=APITimestampFormat.NANOSECOND):
        """ Updates the audit settings """
        return self._xjtrans("/settings/audit", "PUT", config, as_json, timestamp_format)

    def get_cors_domains(self, as_json=True, timestamp_format=APITimestampFormat.NANOSECOND):
        """ Retrieves the cors_domains settings """
        return self._xjtrans("/settings/cors_domains", "GET", None, as_json, timestamp_format)

    def update_cors_domains(self, config, as_json=True, timestamp_format=APITimestampFormat.NANOSECOND):
        """ Updates the cors_domains settings """
        return self._xjtrans("/settings/cors_domains", "PUT", config, as_json, timestamp_format)

    def update_firewall_config(self, config, as_json=True, timestamp_format=APITimestampFormat.NANOSECOND):
        """ Updates the firewall settings """
        return self._xjtrans("/settings/firewall", "POST", config, as_json, timestamp_format)

    def get_firewall_config(self, as_json=True, timestamp_format=APITimestampFormat.NANOSECOND):
        """ Returns the firewall settings """
        return self._xjtrans("/settings/firewall", "GET", None, as_json, timestamp_format)

    def get_raw(self):
        """ Returns the raw settings contained in the configuration file in plain text"""
        resp = self.shark.conn.request("GET", self.uri_prefix + "/settings/raw")
        data = resp.content
        return data

    def update_raw(self, settings):
        self.shark.conn.request("PUT", self.uri_prefix + "/settings/raw",
                                body=settings, extra_headers={'Content-Type' : 'text/plain'})

    def reset_raw(self):
        self.shark.conn.request("PUT", self.uri_prefix + "/settings/raw",
                                body='', extra_headers={'Content-Type' : 'text/plain', 'Content-length' : 0})

    def get_protocol_groups(self):
        """ Returns the protocol_groups settings contained in the configuration file in plain text"""
        resp = self.shark.conn.request("GET", self.uri_prefix + "/settings/protocol_groups")
        data = resp.content
        return data

    def update_protocol_groups(self, settings):
        resp = self.shark.conn.request("PUT", self.uri_prefix + "/settings/protocol_groups",
                                       body=settings, extra_headers={'Content-Type' : 'text/plain'})
        data = resp.content
        return data

    def get_protocol_names(self):
        """ Returns the protocol_names settings contained in the configuration file in plain text"""
        resp = self.shark.conn.request(self.uri_prefix + "/settings/protocol_names", "GET")
        data = resp.read()
        return data

    def update_protocol_names(self, settings):
        resp = self.shark.conn.request("GET", self.uri_prefix + "/settings/protocol_names")
        data = resp.content
        return data

    def get_notification(self, as_json=True, timestamp_format=APITimestampFormat.NANOSECOND):
        """ Returns notification and SMTP settings from management daemon"""
        return self._xjtrans("/settings/notification", "GET", None, as_json, timestamp_format)

    def update_notification(self, config, as_json=True, timestamp_format=APITimestampFormat.NANOSECOND):
        """ Updates notification and SMTP settings on management daemon"""
        return self._xjtrans("/settings/notification", "PUT", config, as_json, timestamp_format)

    def send_test_email(self, config, as_json=True, timestamp_format=APITimestampFormat.NANOSECOND):
        """ Send a test email using the given config"""
        return self._xjtrans("/settings/notification/send_test_mail", "POST", config, as_json, timestamp_format)

    def send_test_trap(self, config, as_json=True, timestamp_format=APITimestampFormat.NANOSECOND):
        return self._xjtrans('/settings/notification/send_test_trap', "POST", config, as_json, timestamp_format)


    def get_profiler_export(self, as_json=True, timestamp_format=APITimestampFormat.NANOSECOND):
        """Get the netprofiler export
        """
        return self._xjtrans('/settings/profiler_export', 'GET', None, as_json, timestamp_format)

    def update_profiler_export(self, params=None, as_json=True, timestamp_format=APITimestampFormat.NANOSECOND):
        """Set the netprofiler export
        """
        return self._xjtrans('/settings/profiler_export', 'PUT', params, as_json, timestamp_format)

class Interfaces(API4Group):
    def get_all(self, params=None, as_json=True, timestamp_format=APITimestampFormat.NANOSECOND):
        """ Lists the interfaces on the system """
        return self._xjtrans("/interfaces", "GET", None, as_json, timestamp_format, params=params)

    def get_details(self, handle, as_json=True, timestamp_format=APITimestampFormat.NANOSECOND):
        """ Retrieves information about an interface """
        return self._xjtrans("/interfaces/%s" % handle, "GET", None, as_json, timestamp_format)

    def update(self, handle, config, as_json=True, timestamp_format=APITimestampFormat.NANOSECOND):
        """ Updates the interface configuration """
        return self._xjtrans("/interfaces/%s" % handle, "PUT", config, as_json, timestamp_format)

    def start_blink(self, handle, as_json=True, timestamp_format=APITimestampFormat.NANOSECOND):
        """ Sets an interface LED to blink mode. """
        return self._xjtrans("/interfaces/%s/blink_status" % handle, "PUT", {"blink_status":"ON"}, as_json, timestamp_format)

    def stop_blink(self, handle, as_json=True, timestamp_format=APITimestampFormat.NANOSECOND):
        """ Sets an interface LED to non-blink mode. """
        return self._xjtrans("/interfaces/%s/blink_status" % handle, "PUT", {"blink_status":"OFF"}, as_json, timestamp_format)

    def create_export(self, handle, config=None, as_json=True, timestamp_format=APITimestampFormat.NANOSECOND):
        """ Create an export and return url """
        return self._xjtrans("/interfaces/%s/exports" % handle, "POST", config, as_json, timestamp_format)

    def get_all_exports(self, handle, as_json=True, timestamp_format=APITimestampFormat.NANOSECOND):
        """ List all exports for this job """
        return self._xjtrans("/interfaces/%s/exports" % handle, "GET", None, as_json, timestamp_format)

    def get_export_details(self, handle, export_id, as_json=True, timestamp_format=APITimestampFormat.NANOSECOND):
        """ Return details for a specific export """
        return self._xjtrans("/interfaces/%s/exports/%s" % (handle, export_id), "GET", None, as_json, timestamp_format)

    def get_packets_from_export(self, handle, export_id, path=None, overwrite=False):
        """ Fetch packets from export ID """
        return self.shark.conn.download(
            self.uri_prefix + "/interfaces/%s/exports/%s/packets" % (handle, export_id),
            path, overwrite=overwrite)

    def get_packets(self, handle, path=None, params=None):
        """ Directly fetch packets for this interface, with optional parameters """
        return self.shark.conn.download(self.uri_prefix + "/interfaces/%s/packets" % handle, path, params=params)

    def delete_export(self, handle, export_id):
        """ Delete an export """
        return self._xjtrans("/interfaces/%s/exports/%s" % (handle, export_id), "DELETE", None, True, APITimestampFormat.NANOSECOND)


class Jobs(API4Group):
    def get_all(self, as_json=True, timestamp_format=APITimestampFormat.NANOSECOND):
        """ Lists the capture jobs on the system """
        return self._xjtrans("/jobs" , "GET", None, as_json, timestamp_format)

    def add(self, config, as_json=True, timestamp_format=APITimestampFormat.NANOSECOND):
        """ Add a new capture job to the system """
        return self._xjtrans("/jobs", "POST", config, as_json, timestamp_format)

    def delete(self, handle):
        """ Updates the capture jobs configuration """
        return self._xjtrans("/jobs/%s" % handle, "DELETE", None, True, APITimestampFormat.NANOSECOND)

    def get_details(self, handle, as_json=True, timestamp_format=APITimestampFormat.NANOSECOND):
        """ Retrieves information about a capture job """
        return self._xjtrans("/jobs/%s" % handle, "GET", None, as_json, timestamp_format)

    def update(self, handle, config, as_json=True, timestamp_format=APITimestampFormat.NANOSECOND):
        """ Updates the capture jobs configuration """
        return self._xjtrans("/jobs/%s" % handle, "PUT", config, as_json, timestamp_format)

    def create_export(self, handle, config=None, as_json=True, timestamp_format=APITimestampFormat.NANOSECOND):
        """ Create an export and return url """
        return self._xjtrans("/jobs/%s/exports" % handle, "POST", config, as_json, timestamp_format)

    def get_all_exports(self, handle, as_json=True, timestamp_format=APITimestampFormat.NANOSECOND):
        """ List all exports for this job """
        return self._xjtrans("/jobs/%s/exports" % handle, "GET", None, as_json, timestamp_format)

    def get_export_details(self, handle, export_id, as_json=True, timestamp_format=APITimestampFormat.NANOSECOND):
        """ Return details for a specific export """
        return self._xjtrans("/jobs/%s/exports/%s" % (handle, export_id), "GET", None, as_json, timestamp_format)

    def delete_export(self, handle, export_id):
        """ Delete an export """
        return self._xjtrans("/jobs/%s/exports/%s" % (handle, export_id), "DELETE", None, True, APITimestampFormat.NANOSECOND)

    def get_packets_from_export(self, handle, export_id, path=None, overwrite=False):
        """ Fetch packets from export ID """
        return self.shark.conn.download(
            self.uri_prefix + "/jobs/%s/exports/%s/packets" % (handle, export_id),
            path, overwrite=overwrite)

    def get_packets(self, handle, path=None, params=None):
        """ Directly fetch packets for this job, with optional parameters """
        return self.shark.conn.download(self.uri_prefix + "/jobs/%s/packets" % handle, path, params=params)

    def state_update(self, handle, config, as_json=True, timestamp_format=APITimestampFormat.NANOSECOND):
        """ Updates the capture jobs status """
        return self._xjtrans("/jobs/%s/status" % handle, "PUT", config, as_json, timestamp_format)

    def get_config(self, handle, as_json=True, timestamp_format=APITimestampFormat.NANOSECOND):
        """ Retrieves configuration information about a capture job """
        return self._xjtrans("/jobs/%s/config" % handle, "GET", None, as_json, timestamp_format)

    def get_status(self, handle, as_json=True, timestamp_format=APITimestampFormat.NANOSECOND):
        """ Retrieves status information about a capture job """
        return self._xjtrans("/jobs/%s/status" % handle, "GET", None, as_json, timestamp_format)

    def get_index(self, handle, as_json=True, timestamp_format=APITimestampFormat.NANOSECOND):
        """ Retrieves index information about a capture job """
        return self._xjtrans("/jobs/%s/index" % handle, "GET", None, as_json, timestamp_format)

    def get_stats(self, handle, as_json=True, timestamp_format=APITimestampFormat.NANOSECOND):
        """ Retrieves statistics about a capture job """
        return self._xjtrans("/jobs/%s/stats" % handle, "GET", None, as_json, timestamp_format)

class Clips(API4Group):
    def get_all(self, as_json=True, timestamp_format=APITimestampFormat.NANOSECOND):
        """ Lists the trace clips on the system """
        return self._xjtrans("/clips" , "GET", None, as_json, timestamp_format)

    def add(self, config, as_json=True, timestamp_format=APITimestampFormat.NANOSECOND):
        """ Add a new trace clip to the system """
        return self._xjtrans("/clips", "POST", config, as_json, timestamp_format)

    def get_details(self, handle, as_json=True, timestamp_format=APITimestampFormat.NANOSECOND):
        """ Retrieves information about a trace clip """
        return self._xjtrans("/clips/%s" % handle, "GET", None, as_json, timestamp_format)

    def update(self, handle, config, as_json=True, timestamp_format=APITimestampFormat.NANOSECOND):
        """ Updates the trace clip configuration """
        return self._xjtrans("/clips/%s" % handle, "PUT", config, as_json, timestamp_format)

    def delete(self, handle):
        """Delete a trace clip."""
        return self._xjtrans('/clips/%s' % handle, "DELETE", None, True, APITimestampFormat.NANOSECOND)

    def set_locked(self, handle, config, as_json=True, timestamp_format=APITimestampFormat.NANOSECOND):
        """ Locks/unlocks the trace clip """
        return self._xjtrans("/clips/%s/status" % handle, "PUT", config, as_json, timestamp_format)

    def create_export(self, handle, config=None, as_json=True, timestamp_format=APITimestampFormat.NANOSECOND):
        """ Create an export and return url """
        return self._xjtrans("/clips/%s/exports" % handle, "POST", config, as_json, timestamp_format)

    def get_all_exports(self, handle, as_json=True, timestamp_format=APITimestampFormat.NANOSECOND):
        """ List all exports for this job """
        return self._xjtrans("/clips/%s/exports" % handle, "GET", None, as_json, timestamp_format)

    def get_export_details(self, handle, export_id, as_json=True, timestamp_format=APITimestampFormat.NANOSECOND):
        """ Return details for a specific export """
        return self._xjtrans("/clips/%s/exports/%s" % (handle, export_id), "GET", None, as_json, timestamp_format)

    def delete_export(self, handle, export_id):
        """ Delete an export """
        return self._xjtrans("/clips/%s/exports/%s" % (handle, export_id), "DELETE", None, True, APITimestampFormat.NANOSECOND)

    def get_packets_from_export(self, handle, export_id, path=None, overwrite=False):
        """ Fetch packets from export ID """
        return self.shark.conn.download(
            self.uri_prefix + "/clips/%s/exports/%s/packets" % (handle, export_id),
            path, overwrite=overwrite)

    def get_packets(self, handle, path=None, params=None):
        """ Directly fetch packets from this clip, with optional parameters """
        return self.shark.conn.download(self.uri_prefix + "/clips/%s/packets" % handle, path, params=params)

    def get_config(self, handle, as_json=True, timestamp_format=APITimestampFormat.NANOSECOND):
        """ Retrieves configuration information about a trace clip """
        return self._xjtrans("/clips/%s/config" % handle, "GET", None, as_json, timestamp_format)

    def get_status(self, handle, as_json=True, timestamp_format=APITimestampFormat.NANOSECOND):
        """ Retrieves status information about a trace clip """
        return self._xjtrans("/clips/%s/status" % handle, "GET", None, as_json, timestamp_format)

    def get_index(self, handle, as_json=True, timestamp_format=APITimestampFormat.NANOSECOND):
        """ Retrieves index information about a trace clip """
        return self._xjtrans("/clips/%s/index" % handle, "GET", None, as_json, timestamp_format)

class Files(API4Group):
    def get_all(self, details = False, recursive= False, as_json=True, timestamp_format=APITimestampFormat.NANOSECOND):
        """ Lists all user files on the system """

        # This call includes some parameters in the url
        url = "/fs"

        params = {}

        if details == True:
            params['details'] = True

        if recursive:
            params['recursive'] = True

        return self._xjtrans(url , "GET", None, as_json, timestamp_format, params=params)

    def upload_raw(self, path, data, headers):
        """Raw wrapper around the file upload. """
        if path[0] == '/':
            path = path[1:]
        return self.shark.conn.upload(self.uri_prefix + "/fs/%s" % path, data, extra_headers=headers)

    def upload_xjobject(self, path, data, headers, as_json=True, timestamp_format = APITimestampFormat.NANOSECOND):
        """Upload an XML or JSON-encoded file object."""
        if path[0] == '/':
            path = path[1:]

        return self._xjtrans("/fs/%s" % path, "POST", data, as_json, timestamp_format, custom_headers = headers)

    def get_details(self, path, params=None, as_json=True, details = False, recursive = False,
                    timestamp_format=APITimestampFormat.NANOSECOND):
        """Retrieves information about a disk file """
        if path[0] == '/':
            path = path[1:]

        # This call includes some parameters in the url
        url = "/fs/%s" % path

        if params is None:
            params = {}

        if details == True:
            params['details'] = True

        if recursive:
            params['recursive'] = True

        return self._xjtrans(url, "GET", None, as_json, timestamp_format, params)

    def upload_trace(self, path, filename, local_file_ref ):
        """Convenience function to upload a trace file."""
        headers = {'Content-Disposition' : filename,
                   'Content-Type' : 'application/octet-stream'}
        if path[0] == '/':
            path = path[1:]
        return self.shark.conn.upload(self.uri_prefix + "/fs/%s" % path, local_file_ref, extra_headers=headers)

    def download(self, path, local_path=None, overwrite=False):
        """Convenience function to download a file."""
        if path[0] == '/':
            path = path[1:]
        return self.shark.conn.download(
            self.uri_prefix + "/fs/%s/download" % path,
            path=local_path, overwrite=overwrite)

    def delete(self, path):
        """Delete a file from the system."""
        if path[0] == '/':
            path = path[1:]
        return self._xjtrans('/fs/%s' % path, "DELETE", None, True, APITimestampFormat.NANOSECOND)

    def update(self, path, config, as_json=True, timestamp_format=APITimestampFormat.NANOSECOND):
        """ Updates the disk file """
        if path[0] == '/':
            path = path[1:]
        return self._xjtrans("/fs/%s" % path, "PUT", config, as_json, timestamp_format)

    def checksum(self, path, as_json=True, timestamp_format=APITimestampFormat.NANOSECOND):
        """ Obtains a disk file checksum """
        if path[0] == '/':
            path = path[1:]
        return self._xjtrans("/fs/%s/checksum" % path, "GET", None, as_json, timestamp_format)

    def move(self, path, config, as_json=True, timestamp_format=APITimestampFormat.NANOSECOND):
        """ Moves a disk file """
        if path[0] == '/':
            path = path[1:]
        return self._xjtrans("/fs/%s/move" % path, "POST", config, as_json, timestamp_format)

    def copy(self, path, config, as_json=True, timestamp_format=APITimestampFormat.NANOSECOND):
        """ Moves a disk file """
        if path[0] == '/':
            path = path[1:]
        return self._xjtrans("/fs/%s/copy" % path, "POST", config, as_json, timestamp_format)

    def create_index(self, path, as_json=True, timestamp_format=APITimestampFormat.NANOSECOND):
        """ Creates an index for a disk file """
        if path[0] == '/':
            path = path[1:]
        return self._xjtrans("/fs/%s/index" % path, "POST", None, as_json, timestamp_format)

    def delete_index(self, path):
        """ Deletes an index for a disk file """
        if path[0] == '/':
            path = path[1:]
        return self._xjtrans("/fs/%s/index" % path, "DELETE", None, True, APITimestampFormat.NANOSECOND)

    def index_info(self, path, as_json=True, timestamp_format=APITimestampFormat.NANOSECOND):
        """ Gets information on an index for a disk file """
        if path[0] == '/':
            path = path[1:]
        return self._xjtrans("/fs/%s/index" % path, "GET", None, as_json, timestamp_format)

    def update_timeskew(self, path, packets, as_json=True, timestamp_format=APITimestampFormat.NANOSECOND):
        """ Creates a timeskew estimation for a multi-segment file """
        url = "/fs/%s/timeskew_estimate" % path

        params = {
            'packet_count': packets
            }

        if path[0] == '/':
            path = path[1:]
        return self._xjtrans(url, "PUT", None, as_json, timestamp_format, params=params)

    def delete_timeskew(self, path):
        """ Deletes a timeskew estimation for a multi-segment file """
        if path[0] == '/':
            path = path[1:]
        return self._xjtrans("/fs/%s/timeskew_estimate" % path, "DELETE", None, True, APITimestampFormat.NANOSECOND)

    def get_timeskew(self, path, as_json=True, timestamp_format=APITimestampFormat.NANOSECOND):
        """ Gets information on a timeskew estimation for a multi-segment file """
        if path[0] == '/':
            path = path[1:]
        return self._xjtrans("/fs/%s/timeskew_estimate" % path, "GET", None, as_json, timestamp_format)

    def create_export(self, path, config=None, as_json=True, timestamp_format=APITimestampFormat.NANOSECOND):
        """ Create an export and return url """
        if path[0] == '/':
            path = path[1:]
        return self._xjtrans("/fs/%s/exports" % path, "POST", config, as_json, timestamp_format)

    def get_all_exports(self, path, as_json=True, timestamp_format=APITimestampFormat.NANOSECOND):
        """ List all exports for this job """
        if path[0] == '/':
            path = path[1:]
        return self._xjtrans("/fs/%s/exports" % path, "GET", None, as_json, timestamp_format)

    def get_export_details(self, path, export_id, as_json=True, timestamp_format=APITimestampFormat.NANOSECOND):
        """ Return details for a specific export """
        if path[0] == '/':
            path = path[1:]
        return self._xjtrans("/fs/%s/exports/%s" % (path, export_id), "GET", None, as_json, timestamp_format)

    def delete_export(self, path, export_id):
        """ Delete an export """
        if path[0] == '/':
            path = path[1:]
        return self._xjtrans("/fs/%s/exports/%s" % (path, export_id), "DELETE", None, True, APITimestampFormat.NANOSECOND)

    def get_packets_from_export(self, path, export_id, local_path=None, overwrite=False):
        """ Fetch packets from export ID """
        if path[0] == '/':
            path = path[1:]
        return self.shark.conn.download(
            self.uri_prefix + "/fs/%s/exports/%s/packets" % (path, export_id),
            local_path, overwrite=overwrite)

    def get_packets(self, path, local_path=None,  params=None):
        """ Directly fetch packets from file on server, with optional parameters """
        if path[0] == '/':
            path = path[1:]
        return self.shark.conn.download(self.uri_prefix + "/fs/%s/packets" % path, local_path, params=params)

class Users(API4Group):
    def get(self, as_json=True, timestamp_format=APITimestampFormat.NANOSECOND):
        """ Lists the users on the system """
        return self._xjtrans("/auth/users" , "GET", None, as_json, timestamp_format)

    def add(self, config, as_json=True, timestamp_format=APITimestampFormat.NANOSECOND):
        """ Add a user to the system """
        return self._xjtrans("/auth/users", "POST", config, as_json, timestamp_format)

    def get_details(self, username, as_json=True, timestamp_format=APITimestampFormat.NANOSECOND):
        """ Retrieves information about a user """
        return self._xjtrans("/auth/users/%s" % username, "GET", None, as_json, timestamp_format)

    def delete(self, username):
        """ Deletes the given user from the system """
        return self._xjtrans("/auth/users/%s" % username, "DELETE", None, True, APITimestampFormat.NANOSECOND)

    def update(self, user, config):
        """Updates user configuration"""
        return self._xjtrans('/auth/users/{0}'.format(user), 'PUT', config, True, APITimestampFormat.NANOSECOND)

class Groups(API4Group):
    def get(self, as_json=True, timestamp_format=APITimestampFormat.NANOSECOND):
        """ Lists the groups on the system """
        return self._xjtrans("/auth/groups" , "GET", None, as_json, timestamp_format)

    def get_details(self, groupname, as_json=True, timestamp_format=APITimestampFormat.NANOSECOND):
        """ Retrieves information about a group on the system """
        return self._xjtrans("/auth/groups/%s" % groupname, "GET", None, as_json, timestamp_format)

    def add(self, config, as_json=True, timestamp_format=APITimestampFormat.NANOSECOND):
        """ Add a group to the system """
        return self._xjtrans("/auth/groups", "POST", config, as_json, timestamp_format)

    def delete(self, groupname):
        """ Deletes the given group from the system """
        return self._xjtrans("/auth/groups/%s" % groupname, "DELETE", None, True, APITimestampFormat.NANOSECOND)


class Licenses(API4Group):
    def get(self, as_json=True, timestamp_format=APITimestampFormat.NANOSECOND):
        """ Lists all licenses in the system """
        return self._xjtrans("/settings/licenses" , "GET", None, as_json, timestamp_format)

    def get_details(self, license_key, as_json=True, timestamp_format=APITimestampFormat.NANOSECOND):
        """ Get details for a specific license key """
        return self._xjtrans("/settings/licenses/%s" % license_key, "GET", None, as_json, timestamp_format)

    def get_status(self, as_json=True, timestamp_format=APITimestampFormat.NANOSECOND):
        """ Get system-wide licensing status """
        return self._xjtrans("/settings/licenses/status" , "GET", None, as_json, timestamp_format)

    def add_license(self, config, as_json=True, timestamp_format=APITimestampFormat.NANOSECOND):
        """ Add new license to the system """
        return self._xjtrans("/settings/licenses", "POST", config, as_json, timestamp_format)

    def delete_license(self, license_key):
        """ Delete a license from the system. """
        return self._xjtrans("/settings/licenses/%s" % license_key, "DELETE", None, True, APITimestampFormat.NANOSECOND)

    def generate_license_req(self, config, as_json=True, timestamp_format=APITimestampFormat.NANOSECOND):
        """ Generate a license request. """
        return self._xjtrans("/settings/licenses/request", "POST", config, as_json, timestamp_format)


class Views(API4Group):
    def add(self, config, as_json=True, timestamp_format=APITimestampFormat.NANOSECOND):
        """Add and run a new view"""
        return self._xjtrans("/views", "POST", config, as_json, timestamp_format)

    def get_all(self, as_json=True, timestamp_format=APITimestampFormat.NANOSECOND):
        """Return the list of all running views"""
        return self._xjtrans("/views" , "GET", None, as_json, timestamp_format)

    def close(self, handle):
        """Close the running view"""
        return self._xjtrans("/views/%s" % handle, "DELETE", None, True, APITimestampFormat.NANOSECOND)

    def get_config(self, handle, as_json=True, timestamp_format=APITimestampFormat.NANOSECOND):
        """Return the configuration of the given view"""
        return self._xjtrans("/views/%s" % handle, "GET", None, as_json, timestamp_format)

    def get_acl(self, handle, as_json=True, timestamp_format=APITimestampFormat.NANOSECOND):
        """Return the access control list of the given view"""
        return self._xjtrans("/views/%s/acl" % handle, "GET", None, as_json, timestamp_format)

    def update_acl(self, handle, as_json=True, timestamp_format=APITimestampFormat.NANOSECOND):
        """Return the access control list of the given view"""
        # XXX/demmer should this be a PUT??
        return self._xjtrans("/views/%s/acl" % handle, "POST", None, as_json, timestamp_format)

    def get_legend(self, handle, output, as_json=True, timestamp_format=APITimestampFormat.NANOSECOND):
        """Return the legend for the given view output"""
        return self._xjtrans("/views/%s/data/%s/legend" % (handle, output), "GET", None, as_json, timestamp_format)

    def get_data(self, handle, output, as_json=True, timestamp_format=APITimestampFormat.NANOSECOND, **params):
        """Return the output for the given view"""
        return self._xjtrans("/views/%s/data/%s" % (handle, output), "GET", None, as_json, timestamp_format, params)

    def get_stats(self, handle, as_json=True, timestamp_format=APITimestampFormat.NANOSECOND):
        """Return the statistics for the given view"""
        return self._xjtrans("/views/%s/stats" % handle, "GET", None, as_json, timestamp_format)

    def get_processor_stats(self, handle, as_json=True, timestamp_format=APITimestampFormat.NANOSECOND):
        """Return the processor statistics for the given view"""
        return self._xjtrans("/debug/handles/%s" % handle, "GET", None, as_json, timestamp_format)

    def create_watch(self, handle, config, as_json=True, timestamp_format=APITimestampFormat.NANOSECOND):
        """Create a new watch on the view"""
        return self._xjtrans("/views/%s/watches" % handle, "POST", config, as_json, timestamp_format)

    def get_watches(self, handle, as_json=True, timestamp_format=APITimestampFormat.NANOSECOND):
        """Obtain the list of watches for a running view"""
        return self._xjtrans("/views/%s/watches" % handle, "GET", as_json, timestamp_format)

    def delete_watch(self, handle, watch_id, as_json=True, timestamp_format=APITimestampFormat.NANOSECOND):
        """Delete a watch from the view"""
        return self._xjtrans("/views/%s/watches/%s" % (handle, watch_id), "DELETE", None, as_json, timestamp_format)

    def get_watch(self, handle, watch_id, as_json=True, timestamp_format=APITimestampFormat.NANOSECOND):
        """Get settings for a watch on a running view"""
        return self._xjtrans("/views/%s/watches/%s" % (handle, watch_id), "GET", None, as_json, timestamp_format)

    def update_watch(self, handle, watch_id, config, as_json=True, timestamp_format=APITimestampFormat.NANOSECOND):
        """Update settings for a watch on a running view"""
        return self._xjtrans("/views/%s/watches/%s" % (handle, watch_id), "PUT", config, as_json, timestamp_format)

    def disable_watch(self, handle, watch_id, as_json=True, timestamp_format=APITimestampFormat.NANOSECOND):
        """Disable a watch on a running view"""
        return self._xjtrans("/views/%s/watches/%s/disable" % (handle, watch_id), "POST", None, as_json, timestamp_format)

    def enable_watch(self, handle, watch_id, as_json=True, timestamp_format=APITimestampFormat.NANOSECOND):
        """Enable a watch on a running view"""
        return self._xjtrans("/views/%s/watches/%s/enable" % (handle, watch_id), "POST", None, as_json, timestamp_format)

    # XXX/demmer lock/unlock/stop

class System(API4Group):
    def get_info(self, as_json=True, timestamp_format=APITimestampFormat.NANOSECOND):
        """Return the system info"""
        return self._xjtrans("/system/info", "GET", None, as_json, timestamp_format)

    def restart(self, config, as_json=True, timestamp_format=APITimestampFormat.NANOSECOND):
        """Restart the probe or reboot the system"""
        return self._xjtrans("/system/restart", "POST", config, as_json, timestamp_format)

    def get_events(self, as_json=True, timestamp_format=APITimestampFormat.NANOSECOND):
        """Return the events list"""
        return self._xjtrans("/debug/events", "GET", None, as_json, timestamp_format)

    def get_sysdump(self, path, config, as_json=True, timestap_format=APITimestampFormat.NANOSECOND):
        """Dump log archive
        """
        return self.shark.conn.download(self.uri_prefix + "/system/sysdump", path, params=config)

class Certificates(API4Group):

    def get(self, as_json=True, timestamp_format=APITimestampFormat.NANOSECOND):
        """ Returns the certificates settings """
        return self._xjtrans("/settings/certificates", "GET", None, as_json, timestamp_format)


    def update_profiler_export_certificate(self, certificate_data, as_json=True, timestamp_format=APITimestampFormat.NANOSECOND):
        """ Set a new NetProfiler Export certificate"""
        return self._xjtrans("/settings/certificates/profiler_export", "PUT", certificate_data, as_json, timestamp_format)

    def generate_profiler_export_certificate(self, certificate_data, as_json=True, timestamp_format=APITimestampFormat.NANOSECOND):
        """ Generate new Trusted NetProfiler certificate"""
        return self._xjtrans("/settings/certificates/profiler_export/generate", "POST", certificate_data, as_json, timestamp_format)

    def copy_web_certificate(self, as_json=True, timestamp_format=APITimestampFormat.NANOSECOND):
        """ Reuses the web certificate as netprofiler export certificate"""
        return self._xjtrans("/settings/certificates/profiler_export/copy_web", "POST", None, as_json, timestamp_format)


    def update_web_certificate(self, certificate_data, as_json=True, timestamp_format=APITimestampFormat.NANOSECOND):
        """ Set a new Web certificate"""
        return self._xjtrans("/settings/certificates/web", "PUT", certificate_data, as_json, timestamp_format)

    def generate_web_certificate(self, certificate_data, as_json=True, timestamp_format=APITimestampFormat.NANOSECOND):
        """ Generate new web certificate"""
        return self._xjtrans("/settings/certificates/web/generate", "POST", certificate_data, as_json, timestamp_format)

    def copy_profiler_export_certificate(self, as_json=True, timestamp_format=APITimestampFormat.NANOSECOND):
        """ Reuses the NetProfiler Export's certificate as web certificate"""
        return self._xjtrans("/settings/certificates/web/copy_profiler_export", "POST", None, as_json, timestamp_format)


    def add_trusted_profiler_certificate(self, certificate_data, as_json=True, timestamp_format=APITimestampFormat.NANOSECOND):
        """ Add a new Trusted NetProfiler certificate"""
        return self._xjtrans("/settings/certificates/trusted_profilers", "POST", certificate_data, as_json, timestamp_format)

    def delete_trusted_profiler_certificate(self, id, as_json=True, timestamp_format=APITimestampFormat.NANOSECOND):
        """ Delete a new Trusted NetProfiler certificate by its id"""
        return self._xjtrans("/settings/certificates/trusted_profilers/"+id, "DELETE", None, as_json, timestamp_format)

class Stats(API4Group):
    def get_memory(self, as_json=True, timestamp_format=APITimestampFormat.NANOSECOND):
        """ Returns the memory stats """
        return self._xjtrans("/stats/memory", "GET", None, as_json, timestamp_format)

    def get_profiler_export(self, as_json=True, timestamp_format=APITimestampFormat.NANOSECOND):
        """ Returns the netprofiler export stats """
        return self._xjtrans("/stats/profiler_export", "GET", None, as_json, timestamp_format)

    def get_storage(self, as_json=True, timestamp_format=APITimestampFormat.NANOSECOND):
        """Returns the storage stats
        """
        return self._xjtrans("/stats/storage", "GET", None, as_json, timestamp_format)

class Info(API4Group):
    def get_fields(self, as_json=True):
        """Returns all the extractor fields
        """
        return self._xjtrans("/fields.json", 'GET', None, as_json)

class Misc(API4Group):
    def ping(self, method="GET", data=None, headers=None):
        """Run the ping command using the specified method and
        optionally passing the given data. Note that for POST/PUT the
        server will echo back exactly what was posted.

        For GET, the content type can be requested using an Accept
        header of either text/xml or application/json.
        """
        return self._xjtrans(self.uri_prefix + "/ping", method, data, True, custom_headers=headers)


class Update(API4Group):
    def get(self):
        return self._xjtrans('/system/update', 'GET', None, True, APITimestampFormat.NANOSECOND)

    def load_iso_from_url(self, data):
        """Download upload iso from an url"""
        return self._xjtrans('/system/update/url', 'POST', data, True, APITimestampFormat.NANOSECOND)

    def upload_iso(self, f):
        """Given a file descriptor `f`, uploads the content to the server as an upload iso
        """
        headers = {'Content-Disposition' : 'update.iso',
                   'Content-Type' : 'application/octet-stream'}

        return self.shark.conn.upload(self.uri_prefix + "/system/update/iso", data=f, extra_headers=headers)

    def delete_iso(self, data):
        """Delete a previously uploaded iso"""
        return self._xjtrans('/system/update/state', 'PUT', data, True, APITimestampFormat.NANOSECOND)

    def update(self, data):
        """Perform update of the netshark

        `init_id` is the id of the image on the server that is ready for update
        """
        return self._xjtrans('/system/update/state', 'PUT',
                             data,
                             True, APITimestampFormat.NANOSECOND)

class Storage(API4Group):
    """Encapsulates the storage informations that can be found in
    Maintenance page in the webui
    """
    def get(self):
        """Gets the storage configuration from the Server
        """
        return self._xjtrans('/system/storage', 'GET', None, True, APITimestampFormat.NANOSECOND)

    def reinitialize(self):
        """Reinitializes the packet storage
        """
        return self._xjtrans('/system/storage/reinitialize', 'POST', None, True, APITimestampFormat.NANOSECOND)

    def format(self, data):
        """Formats packet storage
        """
        return self._xjtrans('/system/format_storage', 'POST', data, True, APITimestampFormat.NANOSECOND )


class API4_0(object):
    version = '4.0'
    common_version = '1.0'
    def __init__(self, netshark):
        self.netshark = netshark
        self.common = Common("/api/common/"+self.common_version, self.netshark)
        self.settings = Settings("/api/shark/"+self.version, self.netshark)
        self.interfaces = Interfaces("/api/shark/"+self.version, self.netshark)
        self.jobs = Jobs("/api/shark/"+self.version, self.netshark)
        self.clips = Clips("/api/shark/"+self.version, self.netshark)
        self.fs = Files("/api/shark/"+self.version, self.netshark)
        self.licenses = Licenses("/api/shark/"+self.version, self.netshark)
        self.certificates = Certificates("/api/shark/"+self.version, self.netshark)
        self.system = System("/api/shark/"+self.version, self.netshark)
        self.view = Views("/api/shark/"+self.version, self.netshark)
        self.stats = Stats("/api/shark/"+self.version, self.netshark)
        self.info = Info('/api/shark/'+self.version+'/info', self.netshark)
        self.users = Users('/api/shark/'+self.version, self.netshark)
        self.groups = Groups('/api/shark/'+self.version, self.netshark)
        self.update = Update('/api/shark/'+self.version, self.netshark)
        self.storage = Storage('/api/shark/'+self.version, self.netshark)

        # For the misc handlers just make them methods of the api class itself
        m = Misc('/api/shark/'+self.version, self.netshark)
        self.ping = m.ping

__all__ = ['API4_0']



#!/usr/bin/env python

# Copyright (c) 2015 Riverbed Technology, Inc.
#
# This software is licensed under the terms and conditions of the MIT License
# accompanying the software ("License").  This software is distributed "AS IS"
# as set forth in the License.



"""
This script can be used to start, stop, delete or clear a Capture Job on a
NetShark appliance.

Use the -l option to list the appliance jobs.
"""
import sys

from steelscript.netshark.core.app import NetSharkApp


class ControlJob(NetSharkApp):
    def add_options(self, parser):
        super(ControlJob, self).add_options(parser)
        parser.add_option('-l', action="store_true", dest="list", default=False,
                          help='print a list of available capture jobs')
        parser.add_option('-d', dest="delete", help='delete a job')
        parser.add_option('-s', dest="start", help='start a job')
        parser.add_option('-e', dest="stop", help='stop a job')
        parser.add_option('-c', dest="clear",
                          help='clear a job. This can only be done when the job is stopped.')

    def main(self):
        done = 0

        if self.options.list:
            for j in self.netshark.get_capture_jobs():
                print j
            done += 1

        if self.options.delete is not None:
            job = self.netshark.get_capture_job_by_name(self.options.delete)
            ans = raw_input('Are you sure you want to delete Job %s [yes/no]: ' % self.options.delete)
            if ans.lower() != 'yes':
                print 'Okay, aborting.'
                sys.exit()
            job.delete()
            print 'Job %s deleted.' % self.options.delete
            done += 1

        if self.options.start is not None:
            job = self.netshark.get_capture_job_by_name(self.options.start)
            job.start()
            print 'Job %s started.' % self.options.start
            done += 1

        if self.options.stop is not None:
            job = self.netshark.get_capture_job_by_name(self.options.stop)
            job.stop()
            print 'Job %s stopped.' % self.options.stop
            done += 1

        if self.options.clear is not None:
            job = self.netshark.get_capture_job_by_name(self.options.clear)
            job.clear()
            done += 1

        if done == 0:
            self.parser.error('nothing to do!')


if __name__ == '__main__':
    ControlJob().run()



# Copyright (c) 2015 Riverbed Technology, Inc.
#
# This software is licensed under the terms and conditions of the MIT License
# accompanying the software ("License").  This software is distributed "AS IS"
# as set forth in the License.


import common
import testscenarios
import pytest
from steelscript.common.exceptions import RvbdHTTPException


class EqualityTest(object):
    def _equality_test(self, saved, settings):
        settings.save()
        self.assertNotEqual(saved, settings.get())
        settings.load(saved)
        settings.save()
        self.assertEqual(saved, settings.get())


class Settings(EqualityTest,
               common.SetUpTearDownMixin,
               testscenarios.TestWithScenarios):
    scenarios = common.config.get('4.0') + common.config.get('5.0')

    def test_basic(self):
        # let's not mess around with changing most of these settings
        # in case of a failed test in the middle for some reason
        basic = self.shark.settings.basic
        basic.get()
        for attr in ('hostname', 'domain', 'primary_dns', 'secondary_dns',
                     'ssh_enabled', 'fips_enabled', 'timezone', 'ntp_config',
                     'mgmt_ports'):
            self.assertTrue(attr in basic.data)

        basic.data['domain'] = 'local'
        basic.save()
        saved = basic.get()
        basic.data['domain'] = 'steelscripttestdomain'
        self._equality_test(saved, basic)

    def test_auth(self):
        auth = self.shark.settings.auth
        auth.get()
        auth.data['local_settings']['min_password_length'] = 0
        auth.data['local_settings']['password_change_history'] = 0
        auth.auth_sequence = ['LOCAL']
        auth.save()
        saved = auth.get()
        auth.data['local_settings']['min_password_length'] = 1
        self._equality_test(saved, auth)

    def test_users(self):
        users = self.shark.settings.users
        saved = users.get()
        try:
            users.delete('steelscripttestuser')
        except:
            #all good we came from a failure of this test from
            #the past
            pass

        users.add('steelscripttestuser', 'test', ['Administrators'])
        users.save()

        #check that user is in the system
        flag = False
        for user in users.data:
            if user['name'] == 'steelscripttestuser':
                flag = True
                break
        self.assertEqual(flag, True)

        users.change_password('steelscripttestuser', 'test1234')
        users.delete('steelscripttestuser')
        users.save()

    def test_groups(self):
        groups = self.shark.settings.groups
        saved = groups.get()

        try:
            groups.delete('steelscriptgroup')
        except:
            #all good we came from a failure of this test from
            #the past
            pass

        groups.add('steelscriptgroup', 'test', ['CAPABILITY_ADMINISTRATOR'])
        groups.save()

        #check that user is in the system
        flag = False
        for group in groups.data:
            if group['name'] == 'steelscriptgroup':
                flag = True
                break
        self.assertEqual(flag, True)

        groups.delete('steelscriptgroup')
        groups.save()

    def test_audit(self):
        audit = self.shark.settings.audit
        audit.get()
        for category in audit.data['audit_categories']:
            print category['min_remote_server_level']
            print category['audit_type']
            print category['min_syslog_level']
            print category['description']
            print category['name']
        audit.data['audit_categories'][0]['name'] = 'test'
        audit.save()
        saved = audit.get()
        audit.data['audit_categories'][0]['name'] = 'AUTHENTICATION'
        self._equality_test(saved, audit)
    
    @pytest.mark.skipif(True, reason='If test fails bad things will happen')
    def test_licenses(self):
        licenses = self.shark.settings.licenses
        saved = licenses.get()
        self.assertTrue(len(saved) > 0)
        licenses.save()
        self.assertTrue(licenses.status() is not None)

        # this will return an "Invalid characters" error
        self.assertRaises(RvbdHTTPException,
                          licenses.add,
                          'INVALID_KEY')

        licenses.save()
        self.assertEqual(saved, licenses.get())

    def test_firewall(self):
        firewall = self.shark.settings.firewall
        saved = firewall.get()
        firewall.data['firewall_enabled'] = False
        for rule in firewall.data['rules']:
            print rule
        #action can be ACCEPT, DROP, LOG_ACCEPT, LOG_DROP
        #protocol can be ALL, TCP, UDP, ICMP
        firewall.data['rules'].append({
                'action': 'ACCEPT',
                'protocol': 'TCP',
                'description': 'steelscript test',
                'dest_port': 12345})
        firewall.data['rules'].append({
                'action': 'DROP',
                'protocol': 'UDP',
                'description': 'steelscript test',
                'dest_port': 12345})
        firewall.data['rules'].append({
                'action': 'LOG_DROP',
                'protocol': 'TCP',
                'description': 'steelscript test',
                'dest_port': 12367})
        self._equality_test(saved, firewall)

    def test_certificates(self):
        certificates = self.shark.settings.certificates
        default_profiler_pem = '''-----BEGIN CERTIFICATE-----
MIIBsTCCARqgAwIBAgIJAOqvgxZRcO+ZMA0GCSqGSIb3DQEBBAUAMA8xDTALBgNV
BAMTBE1henUwHhcNMDYxMDAyMTY0MzQxWhcNMTYwOTI5MTY0MzQxWjAPMQ0wCwYD
VQQDEwRNYXp1MIGfMA0GCSqGSIb3DQEBAQUAA4GNADCBiQKBgQC0e+f4pJY2eSm1
8U579OKJIyxc/sdKXlLOw0zK6SoNu7XNHmNoObNhQV+3PoSbZNyqW3GuZ54EEKUw
G54kDzHu9cZMGEWvNO6syjZZlfBORpklQoNEsNxAkbhTr9DXfloFKiLouDl8E7jB
hMkbKxnpNcfcl+NEuQ8av2QQWp3jfQIDAQABoxUwEzARBglghkgBhvhCAQEEBAMC
BkAwDQYJKoZIhvcNAQEEBQADgYEATnoqJSym+wATLxgb2Ujdy4CY0gawUXHjidaE
ehyejGdw6VhXpf4lP9Q8JfVERjCoroVkiXenVQe/zer7Qf2hiDB/5s02/+8uiEeq
MJpzsSdEYZUSgpyAcws5PDyr2GVFMI3dfPnl28hVavIkR8r05BPDxKbb8Ic6HWpT
CTDPH3w=
-----END CERTIFICATE-----'''
# generated with:
# openssl req -newkey rsa:2048 -new -nodes -x509 -days 3650 \
# -keyout key.pem -out cert.pem
        pem_cert = '''-----BEGIN PRIVATE KEY-----
MIIEvwIBADANBgkqhkiG9w0BAQEFAASCBKkwggSlAgEAAoIBAQDKzwB0jOUITvdn
x5gHYYeRGvSktYtioWyGTnInZmCfmhhlv1G9tIc5BLdXvy3K5I76437HlVRfahkH
yPpxFukv7e2GPb8I8iAAw8ne/EvrSeE/eQZQs1tLFvAyQxxZvS2sEPSEiaAzhSLN
49MYkKOa0cdnQFtoIk2jxLjuXpOs2VFQwmgRbCNUANZg75gD6B2lQDbKAyTEpG4r
4S0AEH/ybuyhCbErFfjgIzlg1gt6DU6Gx19l8XjZYJKXbidC9dxSGvvbKppV3e+A
7ISxK6/qnSBwlBN3V/6dSAYy6iV+3KekuByheSX+ixIjF85PvY8LhY0q0Xj6oCBe
3iPf28/pAgMBAAECggEBAKAGtlVA13e214EL/OnWCyJ0IpXUqicpOHjBbVGpdpR5
AsdGLzmBOTHEhua182wk7K0K1P6m1exzy0xZSUPy0A0BsGo8ToDAjIAN/Tv11/HQ
Weu7k0AaRe76Ko1+ZuBoZaFfv5DVB5Ofedb77ZV0i6Od47mVDoschiazEzkDZ4L6
j5KebQtGqye7ch5cKq3WUR+k2/VNv+N1X1pQGtmsMtP1QPj5xYgealtdsoXA/9gU
GeLSMLCQ4rEjjDV/Fr6Ee6dS3i3kETpRN7G+ufjtO9Xm+/2m50xukKDAuU2hvs+C
H2EE12Q9HcCkc0GQmnUBO8fLPvlIGlo6SiJC3DA2oAECgYEA5oGabRmJROHEdVOm
UadyyTQKx/ANObhCZxDeI2c1cabP6daEzpuJCqMdyUvEvdgBRtf3LbZFbXXc/AQI
MopbmyNY+ytvg4PganBU7FpvwPmVkBY9n9stf+/Q3FLknwifQicBygKuFs8MNPl1
ynDYQFSlRJvqBUCdoobmVkXZKK0CgYEA4T0vlgJnCP0iRVwbSh0WSRYCwv/GIUzA
vxtpHGZxzktsNtbmyxsYZ0yBkSyTLTEn+w32HgdtN2sWS1zY574rtY9R0ShuvLCN
bjz++4xmwuLqpMP8mvW4m8ASmVQqN1OzV7F+R/YjIPBvZimdH7lfXO1WHH8HCbwg
F//TDu7T/60CgYAXFG8D8YSfEwP6w29pyZxirQVPU6ffWaW8cCHt5Y2iXZN/1Gzj
ywsDt6Vp3F5Mq+4ky8cCGrgE58JCsZyogtX9SKLGM2ks/+1eevTl3YBHEDZ7gN6W
vPlnT/nXta0Sh2h61TEGqxIHUp/kRf4XUQk2F8OchQf/kqK1/U/e34uI2QKBgQCg
aCPIkGNymlvay5K/wGFLoXpMBz3CH1gxgcLkr+yiv0IM+BUbVmuVvX2UtwsFpzlS
6Ql7L0zPp9sTxsbOm7ejMLNS4pmilZXTiWsKGF5ispnqx4zRiudzPGHCgpciGeDi
Ngy6EQfJgJHFTyOQIUNR+dBWsPyBKVIt5UoZG2stbQKBgQCKD1rkIcCs1fVJAOAC
BKQLLx7oi+75LUQNNPXdnV+CWLdxZcZGiEP9tIOT6F6cjoSR5F5rj+GpxsKwLRuC
SHYxmV55aV6BvLl8qPHbkAVkctpcVprGZv3KUa6z4rQAOtkrI/HChSFMqPwVdvfK
Vmpm+ktH79qTy/+/RbpBCBeUPg==
-----END PRIVATE KEY-----
-----BEGIN CERTIFICATE-----
MIIDczCCAlugAwIBAgIJALKq8pCt1HYEMA0GCSqGSIb3DQEBBQUAMFAxCzAJBgNV
BAYTAlVTMQswCQYDVQQIDAJDQTEWMBQGA1UEBwwNU2FuIEZyYW5jaXNjbzEcMBoG
A1UECgwTUml2ZXJiZWQgVGVjaG5vbG9neTAeFw0xMzA5MjYwMDIxMTVaFw0yMzA5
MjQwMDIxMTVaMFAxCzAJBgNVBAYTAlVTMQswCQYDVQQIDAJDQTEWMBQGA1UEBwwN
U2FuIEZyYW5jaXNjbzEcMBoGA1UECgwTUml2ZXJiZWQgVGVjaG5vbG9neTCCASIw
DQYJKoZIhvcNAQEBBQADggEPADCCAQoCggEBAMrPAHSM5QhO92fHmAdhh5Ea9KS1
i2KhbIZOcidmYJ+aGGW/Ub20hzkEt1e/LcrkjvrjfseVVF9qGQfI+nEW6S/t7YY9
vwjyIADDyd78S+tJ4T95BlCzW0sW8DJDHFm9LawQ9ISJoDOFIs3j0xiQo5rRx2dA
W2giTaPEuO5ek6zZUVDCaBFsI1QA1mDvmAPoHaVANsoDJMSkbivhLQAQf/Ju7KEJ
sSsV+OAjOWDWC3oNTobHX2XxeNlgkpduJ0L13FIa+9sqmlXd74DshLErr+qdIHCU
E3dX/p1IBjLqJX7cp6S4HKF5Jf6LEiMXzk+9jwuFjSrRePqgIF7eI9/bz+kCAwEA
AaNQME4wHQYDVR0OBBYEFG8e7kWhLY6jfzR3oyHXMZqqx15+MB8GA1UdIwQYMBaA
FG8e7kWhLY6jfzR3oyHXMZqqx15+MAwGA1UdEwQFMAMBAf8wDQYJKoZIhvcNAQEF
BQADggEBAGkX+VFRkzlJI3Xre/LwZp2XRN7+WcaRdUQK9JbXyIKDwfaUbRS8hhmc
MYFpCXKIe0Cov1Yk3wdp7UQswLexrMXdJjMj/vrd9OGdNYdsh5qr2Awzf/BOjfOW
EylWdyX4+Dmriaek71MM4sJpBRVeRURx1UFcNexYF1qmmvb2iKNW/1qza1cMde4i
NBv+qvPK/Ic74bIa9tMlaJw6R6D5Te19g4fBoB3eUVOlk7LH86Yo5iK2TfOmas5L
OV6q8ntDR9dSbhsfUwCgQhJhCXxMKMB0Fvnh/IhFS1KJTo0aN4soJUnShv0di3/N
UvxFJ1fRfr/EH0By7SF/K4COFhhve6M=
-----END CERTIFICATE-----
'''
        saved = certificates.get()
        certificates.set_certificate_for_web(pem_cert)
        certificates.set_certificate_for_profiler_export(pem_cert)
        certificates.use_profiler_export_certificate_for_web()
        certificates.use_web_interface_certificate_for_profiler_export()
        certificates.generate_new_certificate_for_web()
        certificates.generate_new_certificate_for_profiler_export(
            country='IT',
            email="test@test.com",
            locality='Italy',
            organization='Flyscript',
            state="CT",
            days=70)
        certificates.generate_new_certificate_for_profiler_export()
        certificates.remove_profiler_trusted_certificate('default_profiler')
        certificates.add_profiler_trusted_certificate('default_profiler',
                                                      default_profiler_pem)
        certificates.save()

    def test_cors_domain(self):
        cors = self.shark.settings.cors_domain
        saved = cors.get()
        cors.data.append('http://example_domain1.com')
        cors.data.append('http://example_domain2.com')
        cors.save()
        self._equality_test(saved, cors)

    def test_update(self):
        update = self.shark.settings.update
        saved = update.get()
        self.assertTrue(saved is not None)

    #disabling this because it requires all jobs to be turned off
    #and will loose all the packets in jobs

    # def test_storage(self):
    #     storage = self.netshark.settings.storage
    #     saved = storage.get()

    #     storage.reinitialize()

    #     #format will lose all packets
    #     storage.format()


class Settings5Specific(EqualityTest,
                        common.SetUpTearDownMixin,
                        testscenarios.TestWithScenarios):
    scenarios = common.config.get('5.0')

    def test_4_and_5_compatibility(self):
        self.assertRaises(NotImplementedError, self.shark.settings.get_protocol_groups)
        self.assertRaises(NotImplementedError, self.shark.settings.update_protocol_groups)
        self.assertRaises(NotImplementedError, self.shark.settings.get_protocol_names)
        self.assertRaises(NotImplementedError, self.shark.settings.update_protocol_names)

    def test_alerts(self):
        #this test will fail first time in a new netshark. why?
        #because the rest api does not allow to set an empty
        #SMTP server even if SMTP alerts are not enabled
        #and the basic configuration of a newly created netshark
        #has an empyt SMTP configuration. So when i try to
        #restore the configuration the REST api complains
        alerts = self.shark.settings.alerts
        saved = alerts.get()
        alerts.data['mail']['smtp_server_port'] = 25
        alerts.data['mail']['to_address'] = 'test@test.com'
        alerts.data['mail']['from_address'] = 'fromtest@test.com'
        alerts.data['mail']['smtp_server_address'] = 'smtp.test.com'
        #enable SMTP notifications
        alerts.data['notifier']['enabled'] = True
        #enable SNMP notifications
        alerts.data['notifier']['trap_notification_enabled'] = True
        alerts.data['trap']['receivers'].append({
            'community': 'public',
            'version': 'V2C',
            'address': 'trap.test.com'})
        if saved['mail']['smtp_server_address'] == '':
            saved['mail']['smtp_server_address'] = 'test@riverbed.com'
        self._equality_test(saved, alerts)

        #test snmp
        alerts.test_snmp({
            "address": "trap.riverbed.com",
            "community": "public",
            "version": "V2C"})
        #test smtp
        alerts.test_smtp(
            'notexistentsmtp.riverbed.com',
            'nonexistent@riverbed.com',
            'steelscript_test@riverbed.com')

    def test_profiler_export(self):
        profiler_export = self.shark.settings.profiler_export
        saved = profiler_export.get()
        #remove profiler if exists
        for p in profiler_export.data['profilers']:
            if p.get('address') == 'test.com':
                profiler_export.data['profilers'].remove(p)
        #add profiler
        profiler_export.data['profilers'].append({
            'address': 'test.com'
            })
        # dpi sync
        profiler_export.sync_dpi_with_profiler('test.com')
        profiler_export.save()
        #dpi unsync
        profiler_export.unsync_dpi_with_profiler('test.com')
        profiler_export.save()
        #voip_enabled and dpi_enabled for all ports
        for port in profiler_export.data['adapter_ports']:
            port['voip_enabled'] = True
            port['dpi_enabled'] = True
        profiler_export.save()
        #test checks
        self._equality_test(saved, profiler_export)

    def test_snmp(self):
        snmp = self.shark.settings.snmp
        saved = snmp.get()
        snmp.data['description'] = "steelscript test"
        snmp.data['enabled'] = True
        snmp.data['community'] = "public"
        snmp.data['contact'] = 'steelscript@test.com'
        snmp.data['version'] = 'V2C'
        snmp.data['location'] = 'first floor steelscript'
        self._equality_test(saved, snmp)


class Settings4Specific(EqualityTest,
                        common.SetUpTearDownMixin,
                        testscenarios.TestWithScenarios):
    scenarios = common.config.get('4.0')

    def test_alerts(self):
        """They are not supported for 4.0"""
        def will_raise_ex():
            alerts = self.shark.settings.alerts
            alerts.get()

        self.assertRaises(NotImplementedError, will_raise_ex)

    def test_snmp(self):
        """They are not supported for 4.0"""
        def will_raise_ex():
            x = self.shark.settings.snmp
            x.get()

        self.assertRaises(NotImplementedError, will_raise_ex)

    def test_profiler_export(self):
        profiler_export = self.shark.settings.profiler_export
        saved = profiler_export.get()
        #remove netprofiler if exists
        for p in profiler_export.data['profilers']:
            if p.get('address') == 'test.com':
                profiler_export.data['profilers'].remove(p)
        #add netprofiler
        profiler_export.data['profilers'].append({
            'address': 'test.com'})
        profiler_export.save()
        #voip_enabled and dpi_enabled for all ports
        for port in profiler_export.data['adapter_ports']:
            port['voip_enabled'] = True
        profiler_export.save()
        #test checks
        self._equality_test(saved, profiler_export)



# Copyright (c) 2015 Riverbed Technology, Inc.
#
# This software is licensed under the terms and conditions of the MIT License
# accompanying the software ("License").  This software is distributed "AS IS"
# as set forth in the License.


from steelscript.common.app import Application
from steelscript.netshark.core import NetShark


class NetSharkApp(Application):
    """Simple class to wrap common command line parsing"""
    def __init__(self, *args, **kwargs):
        super(NetSharkApp, self).__init__(*args, **kwargs)
        self.netshark = None

    def parse_args(self):
        super(NetSharkApp, self).parse_args()

    def add_positional_args(self):
        self.add_positional_arg('host', 'NetShark hostname or IP address')

    def add_options(self, parser):
        super(NetSharkApp, self).add_options(parser)
        self.add_standard_options()

    def setup(self):
        self.netshark = NetShark(self.options.host, port=self.options.port,
                                 auth=self.auth,
                                 force_version=self.options.api_version)



# Copyright (c) 2015 Riverbed Technology, Inc.
#
# This software is licensed under the terms and conditions of the MIT License
# accompanying the software ("License").  This software is distributed "AS IS"
# as set forth in the License.



"""This file includes all the types that are exposed to the user during
script creation
"""

__all__=['Operation', 'Key', 'Value']

class Operation(object):
    """Enumeration of the possible operations that can be performed on a fields.
    """
    sum = 'SUM'
    avg = 'AVG'
    timeavg = 'TIME_AVG'
    max = 'MAX'
    min = 'MIN'
    none = 'NONE'


class Field(object):
    def __init__(self, field, operation=Operation.sum, autogen=False,
                 key=False, description=None, default_value=None):

        #do str(field) because field can be a Field instance itself.
        field_string = str(field).replace('::', '.')
        self.field = field
        self.field_string = field_string
        self.operation = operation
        self.autogen = autogen
        self.key = key
        self.description = description
        self.default_value = default_value

    def __str__(self):
        return self.field_string

class Key(Field):
    """This class can be used to identify a fields as a key in a view
    """
    def __init__(self, field, description=None, default_value=None):
        super(Key, self).__init__(field, key=True, operation=Operation.none,
                                  description=description, default_value=default_value)

class Value(Field):
    """This class can be used to identify a fields as a value in a view
    """
    def __init__(self, field, operation=Operation.sum, description=None, default_value=None):
        super(Value, self).__init__(field, operation=operation,
                                    description=description, default_value=default_value)



# Copyright (c) 2015 Riverbed Technology, Inc.
#
# This software is licensed under the terms and conditions of the MIT License
# accompanying the software ("License").  This software is distributed "AS IS"
# as set forth in the License.


from steelscript.common.exceptions import *
from steelscript.netshark.core import _source4 as s4


__all__ = ['Interface5', 'Job5' ]


class Job5(s4.Job4):
    @property
    def dpi_enabled(self):
        return self.data['config']['indexing']['dpi_enabled']

    @dpi_enabled.setter
    def dpi_enabled(self, value):
        self.data['config']['indexing']['dpi_enabled'] = value

    def save(self):
        data = self.data['config'].copy()
        state = self.get_status()['state']

        try:
            self.stop()
        except RvbdHTTPException:
            #it's all good, the job was already STOPPED
            pass
        self._api.update(self.id, data)
        if state != "STOPPED":
            self._api.state_update(self.id, {'state': state})


class Interface5(s4.Interface4):

    def save(self):
        if self.shark.model == "vShark":
            self._api.update(self.id, {'name': self.data['name'],
                                       'description': self.data['description']
                                      })
        else:
            # we are in a normal netshark, we have to
            # delete things we cannot modify
            data = self.data.copy()
            del data['interface_components']
            del data['link']
            del data['board']
            del data['is_promiscuous_mode']
            del data['type']
            del data['id']
            self._api.update(self.id, data)
        self.update()

    @s4.Interface4.name.setter
    def name(self, value):
        self.data['name'] = value



# Copyright (c) 2015 Riverbed Technology, Inc.
#
# This software is licensed under the terms and conditions of the MIT License
# accompanying the software ("License").  This software is distributed "AS IS"
# as set forth in the License.


import steelscript.appfwk.apps.report.modules.c3 as c3
from steelscript.appfwk.apps.report.models import Report
import steelscript.appfwk.apps.report.modules.tables as tables

from steelscript.netshark.appfwk.datasources.netshark import \
    NetSharkTable
from steelscript.appfwk.apps.datasource.modules.analysis import \
    FocusedAnalysisTable

#
# Define a NetShark Report and Table
#
report = Report.create('NetShark Microburst Summary', position=10)
report.add_section()

# Summary Microbursts Graph for NetShark
t = NetSharkTable.create(name='MicroburstsTime', duration=1,
                         resolution='1sec', aggregated=False)

t.add_column('time', label='Time', iskey=True,
             extractor='sample_time', datatype='time')

t.add_column('max_microburst_1ms_bits', label='uBurst 1ms',
             extractor='generic.max_microburst_1ms.bits',
             operation='max', units='B')

t.add_column('max_microburst_10ms_bits', label='uBurst 10ms',
             extractor='generic.max_microburst_10ms.bits',
             operation='max', units='B')

t.add_column('max_microburst_100ms_bits', label='uBurst 100ms',
             extractor='generic.max_microburst_100ms.bits',
             operation='max', units='B')

report.add_widget(c3.TimeSeriesWidget, t,
                  'Microburst Timeseries (1s resolution)', width=6)
report.add_widget(tables.TableWidget, t,
                  'Microburst Bits Summary', width=6)


# Detailed Microburst Template Table
# This uses finer grained microburst extractors
z = NetSharkTable.create(name='MicroburstsFocused', aggregated=False)

z.add_column('time', label='Time', iskey=True,
             extractor='sample_time', datatype='time')

z.add_column('max_microburst_10us_bits', label='uBurst 10us',
             extractor='generic.max_microburst_10us.bits',
             operation='max', units='B')

z.add_column('max_microburst_100us_bits', label='uBurst 100us',
             extractor='generic.max_microburst_100us.bits',
             operation='max', units='B')

z.add_column('max_microburst_1ms_bits', label='uBurst 1ms',
             extractor='generic.max_microburst_1ms.bits',
             operation='max', units='B')

# Local Max Microburst detail
a = FocusedAnalysisTable.create(name='max-focused-table',
                                max=True,
                                zoom_duration='1s',
                                zoom_resolution='1ms',
                                tables={'source': t},
                                related_tables={'template': z})
report.add_widget(c3.TimeSeriesWidget, a,
                  'Max Microburst Timeseries (1ms resolution)', width=6)
report.add_widget(tables.TableWidget, a,
                  'Max Microburst Bits Summary', width=6)

# Local Min Microburst detail
a = FocusedAnalysisTable.create(name='min-focused-table',
                                max=False,
                                zoom_duration='1s',
                                zoom_resolution='1ms',
                                tables={'source': t},
                                related_tables={'template': z})
report.add_widget(c3.TimeSeriesWidget, a,
                  'Min Microburst Timeseries (1ms resolution)', width=6)
report.add_widget(tables.TableWidget, a,
                  'Min Microburst Bits Summary', width=6)



#!/usr/bin/env python

# Copyright (c) 2015 Riverbed Technology, Inc.
#
# This software is licensed under the terms and conditions of the MIT License
# accompanying the software ("License").  This software is distributed "AS IS"
# as set forth in the License.


"""
This script lists out and optionally deletes all clips on a NetShark.
"""

import sys

from steelscript.netshark.core.app import NetSharkApp
from steelscript.commands.steel import prompt_yn


class CreateView(NetSharkApp):
    def add_options(self, parser):
        super(CreateView, self).add_options(parser)
        parser.add_option('--force',
                          help='Delete all clips without prompting',
                          default=False)

    def validate_args(self):
        """Validate arguments if needed"""
        super(CreateView, self).validate_args()

    def main(self):
        clips = self.netshark.get_clips(force_refetch=True)

        if clips:
            for i, c in enumerate(clips):
                print '%3d) %s - %s' % (i, c, c.data['config']['description'])

            if not self.options.force:
                if not prompt_yn('Delete all these clips?',
                                 default_yes=False):
                    print 'Okay, exiting.'
                    sys.exit(0)

            for c in clips:
                c.delete()

            print 'Deleted.'
        else:
            print 'No trace clips found on NetShark.'


if __name__ == '__main__':
    CreateView().run()



# Copyright (c) 2015 Riverbed Technology, Inc.
#
# This software is licensed under the terms and conditions of the MIT License
# accompanying the software ("License").  This software is distributed "AS IS"
# as set forth in the License.

import os
from glob import glob

try:
    from setuptools import setup, find_packages, Command
    packagedata = True
except ImportError:
    from distutils.core import setup
    from distutils.cmd import Command
    packagedata = False

    def find_packages(where='steelscript'):
        return [p for p, files, dirs in os.walk(where) if '__init__.py' in files]

from gitpy_versioning import get_version

setup_args = {
    'name':               'steelscript.netshark',
    'namespace_packages': ['steelscript'],
    'version':            get_version(),
    'author':             'Riverbed Technology',
    'author_email':       'eng-github@riverbed.com',
    'url':                'http://pythonhosted.org/steelscript',
    'license':            'MIT',
    'description':        'Python module for interacting with Riverbed NetShark with SteelScript',

    'long_description': '''SteelScript for NetShark
========================

SteelScript is a collection of libraries and scripts in Python and JavaScript for
interacting with Riverbed Technology devices.

For a complete guide to installation, see:

http://pythonhosted.org/steelscript/
    ''',

    'platforms': 'Linux, Mac OS, Windows',

    'classifiers': [
        'Development Status :: 4 - Beta',
        'Intended Audience :: Developers',
        'Intended Audience :: Information Technology',
        'Intended Audience :: System Administrators',
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python :: 2.6',
        'Programming Language :: Python :: 2.7',
        'Topic :: System :: Networking',
    ],

    'packages': find_packages(exclude=('gitpy_versioning',)),

    'data_files': (
        ('share/doc/steelscript/docs/netshark', glob('docs/*')),
        ('share/doc/steelscript/examples/netshark', glob('examples/*')),
    ),

    'install_requires': (
        'steelscript>=0.9.5',
    ),

    'entry_points': {
        'steel.commands': [
            'netshark = steelscript.netshark.commands'
        ],
        'portal.plugins': [
            'netshark = steelscript.netshark.appfwk.plugin:NetSharkPlugin'
        ],
    },
}

if packagedata:
    setup_args['include_package_data'] = True

setup(**setup_args)



# Copyright (c) 2015 Riverbed Technology, Inc.
#
# This software is licensed under the terms and conditions of the MIT License
# accompanying the software ("License").  This software is distributed "AS IS"
# as set forth in the License.



import time
import json
import logging

from steelscript.common import timeutils
from steelscript.common.datastructures import DictObject
from steelscript.netshark.core import _interfaces
from steelscript.netshark.core._class_mapping import path_to_class
from steelscript.netshark.core._api_helpers import APITimestampFormat

logger = logging.getLogger(__name__)


__all__ = ['View4', 'Output4']


def _to_native(string, legend_entry):
    """ convert `string` to an appropriate native type given `legend_entry` """
    if legend_entry['calculation'] == 'AVG':
        string, den = string.split(':', 1)
        denominator = int(den)
    else:
        denominator = 1

    if (legend_entry['type'].startswith('INT')
            or legend_entry['type'].startswith('UINT')
                or legend_entry['type'] in ('TCP_PORT', 'UDP_PORT')):
        if legend_entry['base'] == 'DEC':
            baseval = 10
        elif legend_entry['base'] == 'HEX':
            baseval = 16
        else:
            raise ValueError('do not know how to handle integer base %s' %
                             legend_entry['base'])
        return int(string, baseval) / denominator

    if legend_entry['type'] == 'DOUBLE':
        return float(string) / denominator

    if legend_entry['type'] == 'BOOLEAN':
        if string.lower() == 'false' or string.lower() == '0':
            return 0
        elif string.lower() == 'true' or string.lower() == '1':
            return 1
        else:
            # Booleans can be a count of successes
            return int(string)

    if legend_entry['type'] == 'ABSOLUTE_TIME':
        return timeutils.nsec_string_to_datetime(string)

    if legend_entry['type'] == 'RELATIVE_TIME':
        return float(string) / denominator

    # XXX anything with IPv4 or ETHER?

    return string

class View4(_interfaces.View):
    def __init__(self, shark, handle, config=None, source=None):
        super(View4, self).__init__()

        self.shark = shark
        self.handle = handle
        self.source = source
        self._outputs = {}
        self.timestamp_format = APITimestampFormat.NANOSECOND

        if config is None:
            self.config = shark.api.view.get_config(handle, timestamp_format=self.timestamp_format)
        else:
            self.config = config

        if source is None:
            path = self.config['input_source']['path']
            self.source = path_to_class(shark, path)

    def __repr__(self):
        d = {}
        d['source'] = self.config['input_source']['path']
        if 'info' in self.config and 'title' in self.config['info']:
            d['title'] = self.config['info']['title']

        return '<View ' + ' '.join(['%s="%s"' % (k, d[k]) for k in d.keys()]) + '>'

    @classmethod
    def _create_from_template(cls, shark, source, template, name=None, sync=True, timestamp_format=APITimestampFormat.NANOSECOND):
        #
        # Partial support. Just take the view in JSON format and apply it.
        #
        template_json = json.loads(template)
        # change the source
        template_json['input_source']['path'] = source.source_path
        res = shark.api.view.add(template_json, timestamp_format=timestamp_format)
        handle = res.get('id')
        view = cls(shark, handle, template_json, source)
        if not source.is_live() and sync:
            try:
                view._poll_completion()
                view._postapply()
            except:
                view.close()
                raise

        return view

    @classmethod
    def _create(cls, shark, source, columns, filters, sync=True, name=None,
                cfg_params=None, template=None, charts=None, sampling_time_msec=None,
                timestamp_format=APITimestampFormat.NANOSECOND):

        parsed_columns = list()
        for column in columns:
            xfield = shark.find_extractor_field_by_name(str(column.field))
            column.field = xfield.id
            column.description = column.description or xfield.description
            parsed_columns.append(column)

        if sampling_time_msec is None:
            sampling_time_msec = 1000

        template = {
            'info' : {},
            'processors': [cls._format_columns(parsed_columns)],
            'parameters': {
                'sampling_time_msec' : sampling_time_msec
            },
            'watches': [],
            'input_source': {
                'path': source.source_path,
                }
            }

        if filters is not None:
            template['input_source']['filters'] = filters

        template['input_source'].update(source.source_options)

        if name is not None:
            template['info']['title'] = name

        res = shark.api.view.add(template, timestamp_format=timestamp_format)

        handle = res.get('id')

        view = cls(shark, handle, template, source)
        return cls._process_view(view, source, sync)

    @staticmethod
    def _format_columns(columns):
        res = dict()
        res['keys'] = list()
        res['metrics'] = list()
        for i, column in enumerate(columns):
            key = 'keys' if column.key else 'metrics'
            value = dict([('field', str(column)), ('id', 'c' + str(i))])
            if key == 'metrics':
                value['operation'] = column.operation
            if column.default_value is not None:
                value['default_value'] = column.default_value
            res[key].append(value)
        res['id'] = 'Flyscript_Processor'
        res['outputs'] = [dict(fields=list(dict(id='c' + str(i))
                          for i in range(len(columns))), id='OOUID_Flyscript')]
        return res


    @classmethod
    def _get_all(cls, shark, timestamp_format=APITimestampFormat.NANOSECOND):
        return [v['id'] for v in shark.api.view.get_all(timestamp_format=timestamp_format)]

    def get_timeinfo(self):
        """ Return a dictionary object with details about the time
        range covered by this view.  The returned object has the
        following entries:

        * `start`: the time of the first packet for which data is available
        * `end`: the end time of the last sample
        * `delta`: the sampling time of each sample

        This function adds a delta to the end time of the view provided by netshark
        If you need the timeinfo provided by netshark as it is use _get_timeinfo
        """
        ti = self._get_timeinfo()
        ti.end = ti.end + ti.delta
        return ti

    def _get_timeinfo(self):
        """Return the timeinfo exactly as it comes from netshark
        """
        # check three times before giving up
        count = 0
        timeinfo = None
        while count < 3:
            res = self.shark.api.view.get_stats(self.handle, timestamp_format=self.timestamp_format)
            timeinfo = res.get('time_details')
            if timeinfo['start'] and timeinfo['end']:
                return DictObject(timeinfo)
            else:
                count += 1
                time.sleep(0.5)
        return DictObject(timeinfo)

    def _poll_completion(self):
        while True:
            res = self.shark.api.view.get_stats(self.handle, timestamp_format=self.timestamp_format)
            status = res.get('state')
            if status == "DONE":
                break
            time.sleep(0.5)

    def _postapply(self):
        for processor in self.config['processors']:
            for output in processor['outputs']:
                self._outputs[output['id']] = Output4(self, output['id'])


    def close(self):
        """Close this view on the server (which permanently deletes
        the view plus any associated configuration and output data)."""
        return self.shark.api.view.close(self.handle)

    def is_ready(self):
        """ Returns a boolean indicating whether data can be
        retrieved from this view or not.  If this function returns
        False, the view data is still being computed, its progress
        can be followed with the method get_progress().
        """
        stats = self.shark.api.view.get_stats(self.handle, timestamp_format=self.timestamp_format)

        return stats['state'] == 'DONE'

    def get_progress(self):
        """ For views applied to non-live sources (i.e., to trace clips
        or trace files), returns an integer between 0 and 100 indicating
        the percentage of the packet source that has been processed.
        Output data is not available on the view until this value reaches
        100% """

        stats = self.shark.api.view.get_stats(self.handle, timestamp_format=self.timestamp_format)
        if stats['state'] == 'DONE' or stats['input_size'] == 0:
            return 100
        if stats['input_size'] != 0:
            return int(float(stats['processed_size']) /
                       stats['input_size'] * 100)


class Output4(_interfaces.Output):
    def __init__(self, view, ouid):
        super(Output4, self).__init__()

        self.view = view
        self.id = ouid
        self._legend = self.get_legend()

    def get_legend(self):
        """ Return the legend for this output.  The legend consists of
        an ordered list of entries, one for each column of data in this
        output.  Each entry is a dictionary object with the following
        entries:

        * `name`: A short name for the field
        * `description`: A slightly longer, more descriptive name for the field
        * `field`: the name of the extractor field that produces this column
        * `calculation`: how data from multiple packets in a sample
          is aggregated (e.g., "SUM", "AVG", etc.)

        The following parameters are intended for internal netshark use:

        * `type`
        * `id`
        * `base`
        * `dimension`
        """
        return self.view.shark.api.view.get_legend(self.view.handle, self.id, timestamp_format=self.view.timestamp_format)

    def _get_time_resolution(self):
        if self.view.timestamp_format == APITimestampFormat.SECOND:
            return 1
        elif self.view.timestamp_format == APITimestampFormat.MILLISECOND:
            return 10 ** 3
        elif self.view.timestamp_format == APITimestampFormat.MICROSECOND:
            return 10 ** 6
        elif self.view.timestamp_format == APITimestampFormat.NANOSECOND:
            return 10 ** 9
        else:
            raise ValueError('invalid time format %s' % str(view.timestamp_format))

    def _convert_sample_time(self, sample_timestamp):
        if self.view.timestamp_format == APITimestampFormat.SECOND:
            return timeutils.sec_string_to_datetime(sample_timestamp)
        elif self.view.timestamp_format == APITimestampFormat.MILLISECOND:
            return timeutils.msec_string_to_datetime(sample_timestamp)
        elif self.view.timestamp_format == APITimestampFormat.MICROSECOND:
            return timeutils.usec_string_to_datetime(sample_timestamp)
        elif self.view.timestamp_format == APITimestampFormat.NANOSECOND:
            return timeutils.nsec_string_to_datetime(sample_timestamp)
        else:
            raise ValueError('invalid time format %s' % str(view.timestamp_format))

    def _parse_output_params(self, start=None, end=None, delta=None,
                             aggregated=False, sortby=None,
                             sorttype="descending", fromentry=0, toentry=0):
        """
        These are the operations to do in case of an aggregated call

        ti = view.get_timeinfo()

        NOTE: get_timeinfo is different from _get_timeinfo

        | ss_start* | ss_end* | shark_start | shark_end | shark_delta       |
        |-----------+---------+-------------+-----------+-------------------|
        | None      | None    | ti.start    | ti.start  | ti.end - ti.start |
        | None      | e       | ti.start    | ti.start  | e - ti.start      |
        | s         | None    | s           | s         | ti.end - s        |
        | s         | e       | s           | s         | e - s             |

        (ss == steelscript)
        """
        if aggregated:
            if delta is not None:
                raise ValueError('delta cannot be used with aggregated requests')
            if start is None or end is None:
                # retrieve the timeinfo only if you need it
                ti = self.view.get_timeinfo()

            # normalize to reduce complexity
            start = start or ti.start
            end = end or ti.end
            # now that it's normalized, time for easy math
            delta = end - start
            end = start

        if hasattr(delta, 'seconds'):
            # looks like a timedelta
            delta = ((delta.days * 24 * 3600) + delta.seconds) * self._get_time_resolution()

        if delta is None:
            # default value = 1s
            delta = self._get_time_resolution()

        if start is None:
            start = 0

        if end is None:
            end = 0
        elif not aggregated:
            # this is to remove the delta added by get_timeinfo()
            end -= delta

        params = {
            'start': start,
            'end':   end,
            'delta': delta
            }

        if aggregated:
            params['aggregated'] = True

        if sortby:
            params.update({
                'sortby': 'x' + str(sortby),
                'sorttype': sorttype,
                'fromentry': int(fromentry),
                'toentry': int(toentry)
                })

        return params

    def get_iterdata(self, start=None, end=None, delta=None,
                     aggregated=False,
                     sortby=None, sorttype="descending",
                     fromentry=0, toentry=0):
        """
        Returns an iterator to the output data. This function is ideal for
        sequential parsing of the view data, because it downloads the
        dataset incrementally as it is accessed.

        `start` and `end` are `datetime.datetime` objects representing
        the earliest and latest packets that should be considered.
        If `start` and `end` are unspecified, the start/end of the
        underlying packet source are used.

        `delta` is a `datetime.timedelta` object that can be used to
        override the default data aggregation interval.  If this
        parameter is unspecified, the underlying view sample interval
        (which defaults to 1 second) is used.  If this parameter is
        specified, it must be an even multiple of the underlying
        view sample interval.

        If `aggregated` is True, the parameter `delta` is automatically
        computed to be the full extent of this request (i.e., the difference
        between the effective start and end times).  This is useful if
        you do not care about timeseries data (e.g., if the data from
        this view is to be plotted in a single chart that has no
        time component).

        The `sortby` parameter is one of the fields of the output (x1, x2 ...)

        The `sorttype` can be:
        * `ascending`: the output is sorted from smallest to largest
        * `descending`: the output is sorted largest to smallest

        The `fromentry` parameter represent the first sorted item we want
        to appear in the output.  0 means from the first one.

        The `toentry` parameter represent the last sorted item we want to
        appear in the output.  0 means all of them.
        """
        params = self._parse_output_params(start, end, delta, aggregated, sortby, sorttype, fromentry, toentry)

        res = self.view.shark.api.view.get_data(self.view.handle, self.id, timestamp_format=self.view.timestamp_format, **params)

        samples = res.get('samples')

        # aggregated debug
        logger.debug('get_data params: %s' % params)

        if samples is None:
            return

        for sample in samples:
            if 'vals' not in sample or sample['p'] == 0:
                continue

            sample['t'] = self._convert_sample_time(sample['t'])

            def convert_one(vec):
                return [ _to_native(v, self._legend[i])
                         for i, v in enumerate(vec) ]
            sample['vals'] = [ convert_one(v) for v in sample['vals']]
            yield sample



#!/usr/bin/env python

# Copyright (c) 2015 Riverbed Technology, Inc.
#
# This software is licensed under the terms and conditions of the MIT License
# accompanying the software ("License").  This software is distributed "AS IS"
# as set forth in the License.



"""
This script shows the simplest way to create a custom view on a NetShark and
retrieve its data. It creates a simple top talker view by processing the remote
file specified on the command line (use the shark_info.py script to list the
files on the appliance), and then saves the data on a csv file that can be
opened in excel.
"""

from steelscript.netshark.core.app import NetSharkApp
from steelscript.netshark.core.types import Value, Key
from steelscript.netshark.core.viewutils import write_csv

CSV_FILE_NAME = "result.csv"


class CreateView(NetSharkApp):
    def add_options(self, parser):
        super(CreateView, self).add_options(parser)
        parser.add_option('--file', help='filename to open')

    def validate_args(self):
        """ Ensure columns are included
        """
        super(CreateView, self).validate_args()

        if not self.options.file:
            self.parser.error('Filename of file on NetShark machine required ("--file").')

    def main(self):
        # Open the remote file
        source = self.netshark.get_file(self.options.file)

        # Specify the column list
        columns = [
            # Views are similar in concept to SQL tables, and the one we are
            # creating has only one key, the source IP address. This means that
            # data will be grouped by source IP.
            # Note that we support auto-completion on column names! If you want
            # to try it, put a breakpoint on the line below and then type
            # "sk.columns." in the debugger. That will work in any inteactive
            # python editor as well (e.g. eclipse, bpython, dreampie...).
            Key(self.netshark.columns.ip.src),

            # Each of the rows in the view is going to have a value column
            # containing the amount of bytes.
            Value(self.netshark.columns.generic.bytes),
        ]

        # Create the view
        v = self.netshark.create_view(source, columns)

        # Retrieve the view data.
        # Aggregated=True means that we want the view data in a single big
        # sample, suitable to be represented in a barchart. The default value
        # for aggregated is False and would give us single-second values ready
        # to be charted in a stripchart.
        output = v.get_data(aggregated=True)

        # Save the view data to disk.
        # Note that, in addition to the data, we need to provide the legend,
        #  which is necessary to decode it.
        write_csv(CSV_FILE_NAME, v.get_legend(), output)
        print "View data written to file " + CSV_FILE_NAME

        # Done! We can close the view.
        v.close()


if __name__ == '__main__':
    CreateView().run()



# Copyright (c) 2015 Riverbed Technology, Inc.
#
# This software is licensed under the terms and conditions of the MIT License
# accompanying the software ("License").  This software is distributed "AS IS"
# as set forth in the License.


import functools
import warnings

import json
import copy
import time


def getted(f):
    @functools.wraps(f)
    def wrapper(self, *args, **kwds):
        if self.data is None:
            raise LookupError('You have to get the configuration first via '
                              'the get method')
        return f(self, *args, **kwds)

    return wrapper


class BasicSettingsFunctionality(object):
    """This class is used as a base for all the settings related classes."""

    def __init__(self, api):

        # : Current settings are stored in this property.
        # :
        # : This is updated by calls to :meth:`.get()` and :meth:`.load()`.
        # : The attribute may be modified directly and the resulting
        # : value pushed to the server via :meth:`.save()`.
        self.data = None

        self._api = api

    def _get(self, f, force=False):
        """Gets the configuration calling the f function
        """
        if self.data is None or force is True:
            self.data = f()
        return copy.deepcopy(self.data)

    def get(self, force=False):
        """Gets the configuration from the server

        :param bool force: set to true to ignore a cached copy

        :returns: a dictionary of settings

        """
        return self._get(self._api.get, force)

    @getted
    def _save(self, f):
        """Saves the configuration using the function f"""
        f(self.data)
        self.get(force=True)

    @getted
    def save(self):
        """Save configuration to the server."""
        self._save(self._api.update)

    @getted
    def cancel(self):
        """Cancel pending changes and reload the configuration from server. """
        return self.get(force=True)

    @getted
    def download(self, filename):
        """Download settings and save to a file."""
        data = self.get()
        with open(filename, 'w') as f:
            f.write(json.dumps(data))

    def load(self, path_or_obj, save=True):
        """Load the configuration from a path or dict

        :param path_or_obj: a string representing a path or a
            dict/list representing the configuration

        :param bool save: if true, automatically save to the server
            after load, default to True

        On success, this discards any changes to data.
        """
        if isinstance(path_or_obj, basestring):
            with open(path_or_obj, 'r') as f:
                self.data = json.loads(f.read())
        elif isinstance(path_or_obj, dict):
            self.data = path_or_obj
        elif isinstance(path_or_obj, list):
            self.data = path_or_obj
        else:
            raise ValueError('path_or_obj muth be a filepath, '
                             'a dict or a list')

        if save is True:
            self.save()


class NoBulk(object):
    """Base class to force get of new configuration in not bulk update capable
    settings

    This basically overrides the save method such that it doesn't perform a
    bulk update on the resource but fetches the new data from the server only
    """
    @getted
    def save(self):
        # this mimics settings behaviour
        # for all the resources that do not allow bulk update
        self.get(force=True)


class Basic(BasicSettingsFunctionality):
    """Wrapper class around basic system settings."""

    # definining get/save here in order to do not touch _api4.py
    def get(self, force=False):
        return self._get(self._api.get_basic)

    @getted
    def save(self):
        self._save(self._api.update_basic)


class Auth(BasicSettingsFunctionality):
    """Wrapper class around authentication settings."""

    def get(self, force=False):
        return self._get(self._api.get_auth)

    @getted
    def save(self):
        self._save(self._api.update_auth)


class Audit(BasicSettingsFunctionality):
    """Wrapper class around audit configuration."""

    def get(self, force=False):
        return self._get(self._api.get_audit)

    @getted
    def save(self):
        self._save(self._api.update_audit)


class Licenses(NoBulk, BasicSettingsFunctionality):
    """Wrapper class around license configuration."""

    @getted
    def save(self):
        NoBulk.save(self)
        warnings.warn('Reboot of netshark is needed to apply the new '
                      'configuration')

    @getted
    def add(self, key):
        """Add a license key."""
        # the add wants a list of keys while the
        # delete wants a single key
        self._api.add_license([key])

    @getted
    def remove(self, key):
        """Remove a license key."""
        self._api.delete_license(key)

    @getted
    def clear(self):
        """Clear all license keys."""
        for lic in self.data:
            self._api.delete_license(lic['key'])

    @getted
    def status(self):
        """Return the license status."""
        return self._api.get_status()


class Firewall(BasicSettingsFunctionality):
    """Wrapper class around firewall settings."""

    def get(self, force=False):
        return self._get(self._api.get_firewall_config)

    @getted
    def save(self):
        self._save(self._api.update_firewall_config)


class Certificates(NoBulk, BasicSettingsFunctionality):
    """Wrapper class around the certificates configuration"""

    def _gen_cert_configuration(self, **kwargs):
        return {'issued_to': {
                'country': kwargs.get('country') or 'US',
                'email': kwargs.get('email') or '',
                'locality': kwargs.get('locality') or 'San Francisco',
                'organization': (kwargs.get('organization') or
                                 'Riverbed Technology'),
                'organization_unit': kwargs.get('organization_unit') or '',
                'state': kwargs.get('state') or 'CA'
                },
                'validity': {
                    'days': kwargs.get('days') or 365
                    }
                }

    @getted
    def save(self):
        NoBulk.save(self)
        warnings.warn('Reboot of netshark is needed to apply the new '
                      'configuration')

    @getted
    def use_profiler_export_certificate_for_web(self):
        """Copies netprofiler export certificate and use it for webui"""
        self._api.copy_profiler_export_certificate()

    @getted
    def set_certificate_for_web(self, cert):
        """Given a certificate in PEM format, uploads to the server and
        sets as webui certificate.

        The PEM certificate must contain both private key and CA-signed
        public certificate"""
        self._api.update_web_certificate({'pem': cert})

    @getted
    def generate_new_certificate_for_web(self, country=None, email=None,
                                         locality=None, organization=None,
                                         organization_unit=None,
                                         state=None, days=None):
        """Generates a new certificate for the webui"""
        kwargs = locals()
        kwargs.pop('self')
        self._api.generate_web_certificate(self._gen_cert_configuration(
                                           **kwargs))

    @getted
    def set_certificate_for_profiler_export(self, cert):
        """Give a certificate in PEM format, uploads to the server and sets
        as netprofiler export certificate

        The PEM certificate must contain both private key and CA-signed public
        certificate"""
        self._api.update_profiler_export_certificate({'pem': cert})

    @getted
    def generate_new_certificate_for_profiler_export(self, country=None,
                                                     email=None, locality=None,
                                                     organization=None,
                                                     organization_unit=None,
                                                     state=None, days=None):
        """Generates a new certificate for netprofiler export"""
        kwargs = locals()
        kwargs.pop('self')
        self._api.generate_profiler_export_certificate(
            self._gen_cert_configuration(**kwargs))

    @getted
    def use_web_interface_certificate_for_profiler_export(self):
        """Copies webui certificate and use it for netprofiler export"""
        self._api.copy_web_certificate()

    @getted
    def add_profiler_trusted_certificate(self, name, cert):
        """Adds the given PEM certificate to the list of trusted certificates
        under the given name"""
        self._api.add_trusted_profiler_certificate({'id': name,
                                                    'pem': cert
                                                    })

    @getted
    def remove_profiler_trusted_certificate(self, name):
        """Removes the name of a PEM certificate that is trusted, removes from the
        list of trusted certificates"""
        self._api.delete_trusted_profiler_certificate(name)


class ProfilerExport(BasicSettingsFunctionality):
    """Wrapper class around Profiler Export settings. """

    def get(self, force=False):
        return self._get(self._api.get_profiler_export)

    @getted
    def save(self):
        self._save(self._api.update_profiler_export)


class CorsDomain(BasicSettingsFunctionality):
    """Wrapper class around CORS Domain settings. """

    def get(self, force=False):
        return self._get(self._api.get_cors_domains)

    @getted
    def save(self):
        self._save(self._api.update_cors_domains)


class Users(NoBulk, BasicSettingsFunctionality):
    """Wrapper class around Users configuration settings. """

    @getted
    def add(self, username, password, groups=None, can_be_locked=False):
        """Adds a user to the NetShark

        `username` is a string representing the username

        `groups` is the group the user should be added in.
        Administrators is the administrators group. Add user to that group
        to make the user with administator privileges.

        `can_be_locked` is a boolean representing if the user can be locked out
        from the system or not
        """
        if groups is None:
            groups = []

        self._api.add({'name': username,
                       'password': password,
                       'groups': groups,
                       'can_be_locked': can_be_locked
                       })

    @getted
    def delete(self, username):
        """Delete user from the system

        `username` is the username of the user to be deleted
        """
        self._api.delete(username)

    @getted
    def change_password(self, username, password):
        """Change password of an user
        """
        self._api.update(username, {'existing_password': '',
                                    'new_password': password})


class Groups(NoBulk, BasicSettingsFunctionality):
    """Wrapper class around settings for user groups. """

    @getted
    def add(self, name, description='', capabilities=None):
        """Adds a new group to the system

        :param str name: the name of the group

        :param str description: the description of the group

        :param list capabilities: a list of permissions the group has.
            They can be:

            * CAPABILITY_ADMINISTRATOR,
            * CAPABILITY_APPLY_VIEWS_ON_FILES,
            * CAPABILITY_APPLY_VIEWS_ON_INTERFACES,
            * CAPABILITY_SHARE_VIEWS,
            * CAPABILITY_CREATE_FILES,
            * CAPABILITY_IMPORT_FILES,
            * CAPABILITY_EXPORT_FILES,
            * CAPABILITY_CREATE_JOBS,
            * CAPABILITY_SCHEDULE_WATCHES,
            * CAPABILITY_ACCESS_PROBE_FILES

        """
        if capabilities is None:
            capabilities = []

        self._api.add({'name': name,
                       'description': description,
                       'capabilities': capabilities})

    @getted
    def delete(self, name):
        """Removes group from the groups in the NetShark"""
        self._api.delete(name)


class Update(NoBulk, BasicSettingsFunctionality):
    """Manage system update, ISOs and settings. """

    def load_iso_from_url(self, url):
        """Instruct the NetShark to upload a new ISO from a URL."""
        self._api.load_iso_from_url({'url': url})

    def upload_iso(self, f):
        """Upload a new ISO From a file.

        :param file f: file descriptor to read from

        """
        self._api.upload_iso(f)

    @getted
    def delete_iso(self):
        """Clear any ISO that the NetShark is configured to use."""
        self._api.delete_iso({'state': 'NEUTRAL', 'reset': True})

    @getted
    def update(self):
        """Initiate an update using the configured ISO."""
        if self.data['init_id'] is not '':
            res = self._api.update({'init_id': self.data['init_id'],
                                    'state': 'RUNNING'})
        else:
            msg = ('Server does not have any iso image loaded for upload. '
                   'Upload an iso first and save the configuration to proceed.'
                   )
            raise SystemError(msg)

        while res['state'] == 'RUNNING':
            time.sleep(5)
            res = self.get(force=True)

        if res['state'] != 'NEUTRAL':
            msg = 'Server returned error while update: %s' % res['comment']
            raise SystemError(msg)

        return res


class Storage(NoBulk, BasicSettingsFunctionality):

    def reinitialize(self):
        """Reinitializes the packet storage

        .. warning::
           This operation will lose all packets in every job
        """
        self._api.reinitialize()

        res = self.get(force=True)

        while res['state'] == 'INITIALIZING':
            time.sleep(5)
            res = self.get(force=True)

        if res['state'] != 'OK':
            raise SystemError('Server returned error while reinitializing '
                              'packet storage')

        return res

    def format(self, percentage_reserved_space=0):
        """Formats the packet storage

        .. warning::
           This operation will lose all packets in every job

        :param int percentage_reserved_space: percentage of disk
            reserved starting from the outher boundaries of the disk.

        Since I/O operations at the farmost parts of the disk have
        higher latency this is often used to increase performances of
        the packet recorder.  `percentage_reserved_space` can be any
        value from 0 (default) to 95.

        """

        assert (0 <= percentage_reserved_space < 96)
        self._api.format({'reserved_space': percentage_reserved_space})


class NotImplementedSetting(object):
    def __init__(self, msg=''):
        self.msg = msg

    def get(self, force=True):
        raise NotImplementedError('This setting is not available for this '
                                  'version of NetShark' + self.msg)


class Settings4(object):
    """Interface to various configuration settings on the netshark

    appliance.
    """

    def __init__(self, shark):
        super(Settings4, self).__init__()
        self.shark = shark
        self.auth = Auth(shark.api.settings)
        self.audit = Audit(shark.api.settings)
        self.basic = Basic(shark.api.settings)
        self.licenses = Licenses(shark.api.licenses)
        self.firewall = Firewall(shark.api.settings)
        self.certificates = Certificates(shark.api.certificates)
        self.profiler_export = ProfilerExport(shark.api.settings)
        self.cors_domain = CorsDomain(shark.api.settings)
        self.users = Users(shark.api.users)
        self.groups = Groups(shark.api.groups)
        self.update = Update(shark.api.update)
        self.storage = Storage(shark.api.storage)

        # For the raw text handlers there's nothing that the
        # high-level API needs to add or hide
        self.get_raw = shark.api.settings.get_raw
        self.update_raw = shark.api.settings.update_raw
        self.reset_raw = shark.api.settings.reset_raw

        # these are now DPI in NetShark API 5.0
        self.get_protocol_groups = shark.api.settings.get_protocol_groups
        self.update_protocol_groups = shark.api.settings.update_protocol_groups
        self.get_protocol_names = shark.api.settings.get_protocol_names
        self.update_protocol_names = shark.api.settings.update_protocol_names

        # these have been implemented from API >= 5.0
        self.alerts = NotImplementedSetting()
        self.snmp = NotImplementedSetting()



# Copyright (c) 2017 Riverbed Technology, Inc.
#
# This software is licensed under the terms and conditions of the MIT License
# accompanying the software ("License").  This software is distributed "AS IS"
# as set forth in the License.

import steelscript.appfwk.apps.report.modules.c3 as c3
from steelscript.appfwk.apps.report.models import Report

from steelscript.netshark.appfwk.datasources.netshark_msa import \
    MSATable

#
# Run a latency report against a MSA datasource
#

report = Report.create('NetShark MSA Segment Delay Report', position=10)

report.add_section()

# NetShark Time Series report with min/avg/max
t = MSATable.create(name='MultiSegment Table', resolution='1sec',
                    aggregated=False)
t.add_column('time', label='Time', iskey=True,
             extractor='sample_time', datatype='time')
t.add_column('msa_delay_min', label='Segment Delay (Min)', iskey=False,
             extractor='multi_segment.delay', operation='min')
t.add_column('msa_delay', label='Segment Delay (Avg)', iskey=False,
             extractor='multi_segment.delay', operation='avg')
t.add_column('msa_delay_max', label='Segment Delay (Max)', iskey=False,
             extractor='multi_segment.delay', operation='max')

report.add_widget(c3.TimeSeriesWidget, t, 'MSA Results', width=12)



# Copyright (c) 2015 Riverbed Technology, Inc.
#
# This software is licensed under the terms and conditions of the MIT License
# accompanying the software ("License").  This software is distributed "AS IS"
# as set forth in the License.




"""
This is a set of utility functions for working with views.
"""

from __future__ import absolute_import

import csv
import logging
from datetime import datetime, timedelta
from collections import namedtuple

from steelscript.common.datastructures import DictObject
from steelscript.common.timeutils import tzutc, max_width


class OutputMixer(object):
    """
    Helper class that blends multiple data streams (ie from View.get_data)
    into a single combined stream.
    For example, given a "Bandwidth Over Time" view with separate
    outputs for bytes and packets, this class can be used to create
    a single output stream with bytes and packets columns.

    Mixing is only supported on simple time-based views that do
    not include any keys (e.g., bandwidth over time, etc.)

    See examples/netshark/readview.py for typical usage.
    """

    sourceobj = namedtuple('sourceobj', ['output', 'prefix', 'offset'])

    def __init__(self):
        """
        """
        self._sources = []
        self._legend = []

    def add_source(self, src, prefix=None):
        """ Add new source to mixer

            `src` is time-based view object
        """
        if prefix is None:
            prefix = 'o%d' % len(self._sources)

        obj = self.sourceobj(output=src, prefix=prefix, offset=len(self._legend))
        self._sources.append(obj)

        for field in src.get_legend():
            if field.dimension:
                raise NotImplementedError()

            # create a new record overriding some fields
            entry = DictObject(field)
            entry.id = 'x%d' % len(self._legend)
            entry.name = prefix + entry.name

            self._legend.append(entry)

    def get_legend(self):
        """ Return the legend for each of the source objects
        """
        return self._legend

    def get_iterdata(self, *args, **kwargs):
        """ Return a generator for the combined stream of outputs from each source object
        """
        threshold = timedelta(seconds=1)
        if 'time_thresh' in kwargs:
            threshold = kwargs['time_thresh']
            del kwargs['time_thresh']

        template = [None] * len(self._legend)
        iters = [s.output.get_iterdata(*args, **kwargs) for s in self._sources]
        inputs = [next(i, None) for i in iters]

        # XXX
        infinity = datetime(year=9999, month=12, day=31, tzinfo=tzutc())

        def get_sample_time(s):
            if s is None:
                return infinity
            return s.t

        def min_sample():
            return min(inputs, key=get_sample_time)

        ms = min_sample()
        sample_time = ms.t
        vals = list(template)
        while ms is not None:
            i = inputs.index(ms)
            inputs[i] = next(iters[i], None)

            delta = ms.t - sample_time
            if delta >= threshold:
                yield DictObject.create_from_dict(dict(t=sample_time,
                                                       vals=[vals],
                                                       processed_pkts=None,
                                                       unprocessed_pkts=None))

                sample_time = ms.t
                vals = list(template)

            assert len(ms.vals) == 1

            V = ms.vals[0]
            off = self._sources[i].offset
            for j in range(len(V)):
                vals[off + j] = V[j]

            ms = min_sample()

        yield DictObject.create_from_dict(dict(t=sample_time,
                                               processed_pkts=None,
                                               unprocessed_pkts=None,
                                               vals=[vals]))


def print_data(legend, stream, timeformat='%Y/%m/%d %H:%M:%S.%f',
               include_sample_times=True,
               widths=None, limit=None, line_prefix=''):
    """
    Print the data of a given view output to stdout.

    `widths` is an optional list of integers, specifying how
    many characters wide each column should be.  If it is not
    specified, reasonable defaults are chosen.

    If `limit` is specified, only the first `limit` rows are printed.

    `line_prefix` is a string that is printed at the start of every line.
    """

    labels = []
    if include_sample_times:
        labels.append('Time')
    labels += [f['name'] for f in legend]

    def get_width(field):
        if field['type'] in ('INT16', 'UINT16', 'TCP_PORT', 'UDP_PORT'):
            return 5
        if field['type'] in ('INT32', 'UINT32'):
            return 10
        if field['type'] in ('INT64', 'UINT64', 'RELATIVE_TIME'):
            return 20
        if field['type'] in ('FLOAT', 'DOUBLE'):
            return 20
        if field['type'] == 'BOOLEAN':
            return 5
        if field['type'] == 'IPv4':
            return 15
        if field['type'] in ('STRING', 'SHORT_STRING'):
            # TODO should this be smaller?
            return 64
        if field['type'] == 'ABSOLUTE_TIME':
            return max_width(timeformat)

        logging.warn('NetShark field %s has unknown max_width, guessing 10' %
                     field['type'])
        return 10

    if widths is None:
        widths = []
        if include_sample_times:
            widths.append(max_width(timeformat))

        widths += [get_width(field) for field in legend]

    justified = [label.ljust(widths[i]) for i, label in enumerate(labels)]
    print line_prefix + '  '.join(justified)

    total = 0
    for s in stream:
        if 'gap_start' in s or 'gap_end' in s:
            continue

        base = [s['t'].strftime(timeformat).ljust(widths[0])]
        for vec in s['vals']:
            fields = base + [str(v).ljust(widths[i + 1]) for i, v in enumerate(vec)]

            print line_prefix + '  '.join(fields)

        total += 1
        if total == limit:
            return


def write_csv(filename, legend, stream, include_column_names=True, include_sample_times=True):
    """
    Saves the data of a view output to a comma separated values (csv) file.

    `legend` is an output legend, typically just the result
    of `output.get_legend()`

    `stream` is a series of data samples, typically the result
    of `output.get_data()` or `output.get_iterdata()`

    If `include_column_names` is True, the first line in the file
    will be a summary row indicating the fileds that the file contains.

    If `include_sample_times` is True, the first column will be a
    timestamp.
    """

    ofile = open(filename, "wb")
    writer = csv.writer(ofile)

    if include_column_names:
        labels = []
        if include_sample_times:
            labels.append('Time')

            labels += [ f['name'] for f in legend ]
        writer.writerow(labels)

    for s in stream:
        for v in s['vals']:
            sample = []
            if include_sample_times:
                sample.append(s['t'])

            sample += [str(f) for f in v]

            writer.writerow(sample)

    ofile.close()


class Cursor(object):
    """Given a live view returns only new samples for each get_data call
    """
    def __init__(self, output):
        self.output = output
        self._last_end = 0

    def get_data(self):

        source = self.output.view.source
        view = self.output.view

        ti = view.get_timeinfo()

        start = self._last_end

        data = self.output.get_data(start=start, end=ti.end)

        self._last_end = ti.end

        return data



#!/usr/bin/env python

# Copyright (c) 2015 Riverbed Technology, Inc.
#
# This software is licensed under the terms and conditions of the MIT License
# accompanying the software ("License").  This software is distributed "AS IS"
# as set forth in the License.



"""
This script can be used to export packets from Trace Files, Capture Jobs or
Trace clips on a NetShark appliance. An optional IP address can be specified to
restrict the exported packets to the ones of a single host.

Note: in order to export a clip with this script, you need to make sure the
clip has been given a name
"""

from steelscript.netshark.core.app import NetSharkApp
from steelscript.netshark.core.filters import NetSharkFilter


class ExportApp(NetSharkApp):
    def add_options(self, parser):
        super(ExportApp, self).add_options(parser)
        parser.add_option('--filename', dest="filename", default=None,
                            help='export a Trace File')
        parser.add_option('--jobname', dest="jobname", default=None,
                            help='export a Capture Job')
        parser.add_option('--clipname', dest="clipname", default=None,
                            help='export a Trace Clip')

    def main(self):

        # Do the export based on the specified object type
        if self.options.filename is not None:
            # find the file
            f = self.netshark.get_file(self.options.filename)

            # extract the file name from the full path
            filename = str(f).split('/')[-1]

            # export the file
            f.download(filename)
        elif self.options.jobname is not None:
            # find the job
            job = self.netshark.get_capture_job_by_name(self.options.jobname)

            job.download(job.id + '.pcap')
        elif self.options.clipname is not None:
            # find the clip
            clip = self.netshark.get_trace_clip_by_description(self.options.clipname)

            # extract the clip
            clip.download(clip.id + '.pcap')
        else:
            print 'No options provided ... use "-h" to see available options.'

if __name__ == '__main__':
    ExportApp().run()



# Copyright (c) 2015 Riverbed Technology, Inc.
#
# This software is licensed under the terms and conditions of the MIT License
# accompanying the software ("License").  This software is distributed "AS IS"
# as set forth in the License.

from steelscript.appfwk.apps.report.models import Report
import steelscript.appfwk.apps.report.modules.tables as tables

from steelscript.netshark.appfwk.datasources.netshark import \
    NetSharkJobsTable

report = Report.create('NetShark Capture Jobs', position=10)

report.add_section()

# Table for NetShark
t = NetSharkJobsTable.create(name='Netshark Capture Jobs', cacheable=False)

t.add_column('netshark', label='NetShark', datatype='string', iskey=True)
t.add_column('job_name', label='Job Name', datatype='string')
t.add_column('job_id', label='Job ID', datatype='string', iskey=True)

t.add_column('interface', label='Interface', datatype='string')
t.add_column('bpf_filter', label='BPF Filter', datatype='string')
t.add_column('dpi_enabled', label='Enable DPI', datatype='string')
t.add_column('index_enabled', label='Enable Indexing', datatype='string')
t.add_column('state', label='Status', datatype='string')

t.add_column('start_time', label='Start Time', datatype='string')
t.add_column('end_time', label='End Time', datatype='string')

t.add_column('size', label='Packet Capture Size (Bytes)', datatype='integer')

t.add_column('last_sec_written', label='Last Second Written (Bytes)',
             datatype='integer')
t.add_column('last_min_written', label='Last Minute Written (Bytes)',
             datatype='integer')
t.add_column('last_hr_written', label='Last Hour Written (Bytes)',
             datatype='integer')
t.add_column('last_sec_dropped', label='Last Second Dropped (Bytes)',
             datatype='integer')
t.add_column('last_min_dropped', label='Last Minute Dropped (Bytes)',
             datatype='integer')
t.add_column('last_hr_dropped', label='Last Hour Dropped (Bytes)',
             datatype='integer')

report.add_widget(tables.TableWidget, t, 'netshark-jobs', width=12,
                  height=0, searching=True)
